import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit import rg

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestRG(unittest.TestCase):
    def test_center(self):
        self.assertEqual(rg.CENTER_POINT, (9, 9))

    def test_types_invalid(self):
        self.assertEqual(rg.loc_types((-1, 9)), set(['invalid']))

    def test_types_spawn(self):
        self.assertEqual(rg.loc_types((3, 4)), set(['normal', 'spawn']))

    def test_types_obstacle(self):
        self.assertEqual(rg.loc_types((16, 4)), set(['normal', 'obstacle']))

    def test_types_normal(self):
        self.assertEqual(rg.loc_types((7, 2)), set(['normal']))

    def test_around(self):
        filtered = set(rg.locs_around((3, 3)))
        self.assertEqual(filtered, set([(3, 2), (4, 3), (3, 4), (2, 3)]))

    def test_around_filter(self):
        filtered = set(rg.locs_around((3, 3), filter_out=['obstacle']))
        self.assertEqual(filtered, set([(4, 3), (3, 4)]))

    def test_toward(self):
        self.assertTrue(rg.toward((5, 13), (8, 3)) in [(5, 12), (6, 13)])

    def test_toward_obstacle(self):
        self.assertEqual(rg.toward((5, 2), (4, 3)), (5, 3))



import unittest
from os import path
# TODO: Make this hack better -_-.
try:
    import subprocess as runcommand
except:
    import subprocess as runcommand


class TestAttack(unittest.TestCase):
    def test_acceptance_cli(self):
        testdir = path.dirname(__file__)

        # test if -h runs
        cmd = testdir + "/../rgkit/run.py -h"
        exit_code, ignore = runcommand.getstatusoutput(cmd)
        self.assertEqual(exit_code, 0)

        robot_file = testdir + "/acceptance_robot.py"
        # test simple robot
        cmd = testdir + "/../rgkit/run.py -H {0} {0}".format(robot_file)

        exit_code, out = runcommand.getstatusoutput(cmd)
        self.assertEqual(exit_code, 0)




import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit.gamestate import GameState

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


def delta_for(deltas, loc):
    for d in deltas:
        if d.loc == loc:
            return d


class TestDelta(unittest.TestCase):
    def test_suicide(self):
        state = GameState()
        loc1 = (10, 11)
        dest1 = (10, 10)
        loc2 = (9, 10)

        state.add_robot(loc1, 0)
        state.add_robot(loc2, 1)

        actions = {
            loc1: ['move', dest1],
            loc2: ['suicide']
        }
        deltas = state.get_delta(actions)

        d_guard = delta_for(deltas, loc1)
        d_suicide = delta_for(deltas, loc2)

        self.assertEqual(d_guard.hp - d_guard.hp_end, d_suicide.damage_caused)

    def test_attack(self):
        state = GameState()
        loc1 = (10, 11)
        dest1 = (10, 10)
        loc2 = (9, 10)
        dest2 = dest1

        state.add_robot(loc1, 0)
        state.add_robot(loc2, 1)

        actions = {
            loc1: ['move', dest1],
            loc2: ['attack', dest2]
        }
        deltas = state.get_delta(actions)

        d_move = delta_for(deltas, loc1)
        d_attack = delta_for(deltas, loc2)

        self.assertEqual(d_move.hp - d_move.hp_end, d_attack.damage_caused)

    def test_simple_collision(self):
        state = GameState()
        loc1 = (10, 11)
        dest1 = (10, 10)
        loc2 = (9, 10)
        dest2 = dest1

        state.add_robot(loc1, 0)
        state.add_robot(loc2, 1)

        actions = {
            loc1: ['move', dest1],
            loc2: ['move', dest2]
        }

        deltas = state.get_delta(actions)

        d_move1 = delta_for(deltas, loc1)
        d_move2 = delta_for(deltas, loc2)

        self.assertEqual(d_move1.hp - d_move1.hp_end, d_move2.damage_caused)
        self.assertEqual(d_move2.hp - d_move2.hp_end, d_move1.damage_caused)

    def test_three_collision(self):
        state = GameState()
        loc1 = (10, 11)
        loc2 = (9, 10)
        loc3 = (11, 10)

        dest = (10, 10)

        state.add_robot(loc1, 0)
        state.add_robot(loc2, 1)
        state.add_robot(loc3, 1)

        actions = {
            loc1: ['move', dest],
            loc2: ['move', dest],
            loc3: ['move', dest]
        }

        deltas = state.get_delta(actions)

        d_player0 = delta_for(deltas, loc1)
        d_player1_1 = delta_for(deltas, loc2)
        d_player1_2 = delta_for(deltas, loc3)

        self.assertEqual(d_player0.hp - d_player0.hp_end,
                         d_player1_1.damage_caused + d_player1_2.damage_caused)

        player1_hp_lose = d_player1_1.hp - d_player1_1.hp_end + \
            d_player1_2.hp - d_player1_2.hp_end

        self.assertEqual(player1_hp_lose, d_player0.damage_caused)

    def test_collision_and_attack(self):
        state = GameState()
        loc1 = (10, 11)
        loc2 = (9, 10)
        loc3 = (11, 11)

        dest = (10, 10)
        attack_dest = loc1

        state.add_robot(loc1, 0)
        state.add_robot(loc2, 1)
        state.add_robot(loc3, 1)

        actions = {
            loc1: ['move', dest],
            loc2: ['move', dest],
            loc3: ['attack', attack_dest]
        }

        deltas = state.get_delta(actions)

        d_player0 = delta_for(deltas, loc1)
        d_player1_move = delta_for(deltas, loc2)
        d_player1_attack = delta_for(deltas, loc3)

        self.assertEqual(d_player1_move.damage_caused,
                         settings.collision_damage)

        self.assertTrue(settings.attack_range[0] <=
                        d_player1_attack.damage_caused <=
                        settings.attack_range[1])

        self.assertEqual(d_player0.hp - d_player0.hp_end,
                         d_player1_move.damage_caused +
                         d_player1_attack.damage_caused)

        player1_hp_lose = d_player1_move.hp - d_player1_move.hp_end + \
            d_player1_attack.hp - d_player1_attack.hp_end

        self.assertEqual(player1_hp_lose, d_player0.damage_caused)

    def test_spawn(self):
        state = GameState()
        deltas = state.get_delta(actions={}, spawn=True)
        for d in deltas:
            self.assertTrue(hasattr(d, 'damage_caused'))





import sys
try:
    import tkinter
except ImportError:
    try:
        import tkinter as Tkinter
    except ImportError:
        sys.exit(
            "\033[1;31m"+'rgrun has problem importing tkinter, '
            'which rgrun GUI is dependent on by default. \n'
            'Tkinter is likely not installed or '
            'configured incorrectly. \n'
            'Alternatively, you can try running rgrun as '
            'headless (-H) or use text base UI (-C).'+"\033[0m")

import math

from rgkit.settings import settings
from rgkit.render.settings import settings as render_settings
from rgkit.render.robotsprite import RobotSprite
from rgkit.render.highlightsprite import HighlightSprite
from rgkit.render.utils import millis, rgb_to_hex


class Render(object):
    def __init__(self, game_inst, animations, names=["Red", "Blue"]):
        self.size_changed = False
        self.init = True

        self.cell_border_width = 2
        self.info_frame_height = 100
        self.board_margin = 0

        self._animations = animations
        self._blocksize = 25
        self._winsize = (
            self._blocksize * settings.board_size + self.board_margin)
        self._game = game_inst
        self._paused = True
        self._names = names
        self._layers = {}

        self._master = tkinter.Tk()
        self._master.configure(background="#333")
        self._master.title('Robot Game')

        width = self._winsize
        height = self._winsize

        self._board_frame = tkinter.Frame(self._master, background="#555")
        self._info_frame = tkinter.Frame(self._master, background="#333")
        self._control_frame = tkinter.Frame(self._info_frame)

        self._board_frame.pack(
            side=tkinter.TOP, fill=tkinter.BOTH, expand=True)

        # Tkinter problem: 'pack' distributes space according to which widgets
        # have expand set to true, not to which directions they can actually
        # expand to.
        self._info_frame.pack(side=tkinter.BOTTOM)

        self._control_frame.pack(side=tkinter.RIGHT)

        # Tkinter problem: highlightthickness supposedly defaults to 0, but
        # doesn't.
        self._win = tkinter.Canvas(
            self._board_frame, width=width, height=height,
            background="#555", highlightthickness=0)
        self._info = tkinter.Canvas(
            self._info_frame, width=300, height=self.info_frame_height,
            background="#333", highlightthickness=0)

        self._win.pack()
        self._info.pack(side=tkinter.LEFT)

        self._master.bind('<Configure>', self.on_resize)

        self._labelred = self._info.create_text(
            self._blocksize // 2, self._blocksize * 1 // 4,
            anchor='nw', font='TkFixedFont', fill='#ff1a1a')
        self._labelblue = self._info.create_text(
            self._blocksize // 2, self._blocksize * 7 // 8,
            anchor='nw', font='TkFixedFont', fill='#3455ff')
        self._label = self._info.create_text(
            self._blocksize // 2, self._blocksize * 3 // 2,
            anchor='nw', font='TkFixedFont', fill='white')
        self.create_controls(self._info, width, height)

        self._turn = 1.0
        self._sub_turn = 0.0

        self._highlighted = None
        self._highlighted_target = None

        # Animation stuff
        self._sprites = []
        self._highlight_sprite = None
        self._t_frame_start = 0
        self._t_next_frame = 0
        self._t_cursor_start = 0
        self._slider_delay = 0
        self.update_frame_start_time()

        self.draw_background()
        self.update_info_frame()
        self.update_sprites_new_turn()
        self.paint()

        self.callback()

        self._master.mainloop()

    def on_resize(self, event):
        self.size_changed = True

    def remove_object(self, obj):
        if obj is not None:
            self._win.delete(obj)

    def turn_changed(self):
        if render_settings.clear_highlight_between_turns:
            self._highlighted = None
        if render_settings.clear_highlight_target_between_turns:
            self._highlighted_target = None
        self.update_sprites_new_turn()
        self.update_info_frame()

    def update_block_size(self):
        if self.size_changed:
            if self.init:
                self.init = False
                return

            self.size_changed = False
            self._win.delete("all")

            self._winsize = min(self._board_frame.winfo_width(),
                                self._board_frame.winfo_height())
            self._winsize = max(min(self._winsize,
                                    self._master.winfo_height() -
                                    self.info_frame_height),
                                250)

            self._blocksize = ((self._winsize - self.board_margin) //
                               settings.board_size)
            self._win.configure(width=self._winsize, height=self._winsize)

            self.draw_background()
            self.update_sprites_new_turn()
            self.paint()

    def set_turn(self, new_turn):
        if not self._paused:
            self.toggle_pause()

        self._turn = min(max(new_turn, 1.0), settings.max_turns)
        self._sub_turn = 0.0
        self.update_frame_start_time()
        self.turn_changed()
        self.update_info_frame()
        self.paint()

    def step_turn(self, step):
        self.set_turn(self.current_turn_int() + step)

    def toggle_pause(self):
        self._paused = not self._paused
        # Print "paused" if self._paused else "unpaused".
        self._toggle_button.config(
            text='\u25B6' if self._paused else '\u25FC')
        now = millis()
        if self._paused:
            self._sub_turn = 0.0
        else:
            self.update_frame_start_time(now)

    def update_frame_start_time(self, tstart=None):
        tstart = tstart or millis()
        self._t_frame_start = tstart

        self._t_next_frame = tstart + self._slider_delay

    def create_controls(self, win, width, height):
        def prev():
            self.step_turn(-1)

        def next():
            self.step_turn(+1)

        def restart():
            self.set_turn(1)

        def pause():
            self.toggle_pause()

        def onclick(event):
            x = (event.x - self.board_margin // 2) // self._blocksize
            y = (event.y - self.board_margin // 2) // self._blocksize
            loc = (x, y)
            if (0 <= x < settings.board_size and
                    0 <= y < settings.board_size):
                if loc == self._highlighted:
                    self._highlighted = None
                else:
                    self._highlighted = loc
                action = self._game.get_actions_on_turn(
                    self.current_turn_int()).get(loc)
                if action is not None:
                    self._highlighted_target = action.get("target", None)
                else:
                    self._highlighted_target = None
                self.update_highlight_sprite(True)
                self.update_info_frame()
                self._t_cursor_start = millis()

        self._win.bind("<Button-1>", lambda e: onclick(e))
        self._master.bind('<Left>', lambda e: prev())
        self._master.bind('<Right>', lambda e: next())
        self._master.bind('<space>', lambda e: pause())

        self.show_arrows = tkinter.BooleanVar()
        self.show_arrows.set(True)

        frame = self._control_frame

        arrows_box = tkinter.Checkbutton(
            frame, text="Show Arrows", variable=self.show_arrows,
            command=self.paint)
        arrows_box.pack()

        self._toggle_button = tkinter.Button(
            frame, text='\u25B6', command=pause)
        self._toggle_button.pack(side='left')

        prev_button = tkinter.Button(frame, text='<', command=prev)
        prev_button.pack(side='left')

        next_button = tkinter.Button(frame, text='>', command=next)
        next_button.pack(side='left')

        restart_button = tkinter.Button(frame, text='<<', command=restart)
        restart_button.pack(side='left')

        self._time_slider = tkinter.Scale(
            frame,
            from_=-render_settings.turn_interval // 2,
            to_=render_settings.turn_interval // 2,
            orient=tkinter.HORIZONTAL,
            borderwidth=0,
            length=90)
        self._time_slider.pack(fill=tkinter.X)
        self._time_slider.set(0)

    def draw_grid_object(self, loc, shape="square", layer=0, **kargs):
        layer_id = 'layer %d' % layer
        self._layers[layer_id] = None
        tags = kargs.get("tags", [])
        tags.append(layer_id)
        kargs["tags"] = tags
        x, y = self.grid_to_xy(loc)
        rx, ry = self.square_bottom_corner((x, y))
        if shape == "square":
            item = self._win.create_rectangle(
                x, y, rx, ry,
                **kargs)
        elif shape == "circle":
            item = self._win.create_oval(
                x, y, rx, ry,
                **kargs)
        return item

    def update_layers(self):
        for layer in self._layers:
            self._win.tag_raise(layer)

    def draw_text(self, loc, text, color=None):
        layer_id = 'layer %d' % 9
        self._layers[layer_id] = None
        x, y = self.grid_to_xy(loc)
        item = self._win.create_text(
            x + (self._blocksize - self.cell_border_width) // 2,
            y + (self._blocksize - self.cell_border_width) // 2,
            text=text, font='TkFixedFont', fill=color, tags=[layer_id])
        return item

    def draw_line(self, src, dst, offset=(0, 0), layer=0, **kargs):
        layer_id = 'layer %d' % layer
        self._layers[layer_id] = None
        tags = kargs.get("tags", [])
        tags.append(layer_id)
        kargs["tags"] = tags
        ox, oy = offset
        srcx, srcy = self.grid_to_xy(src)
        dstx, dsty = self.grid_to_xy(dst)

        item = self._win.create_line(
            srcx + ox, srcy + oy, dstx + ox, dsty + oy, **kargs)
        return item

    def update_info_frame(self):
        display_turn = self.current_turn_int()
        display_state = self._game.get_state(display_turn)

        scores = display_state.get_scores()
        red_text = '%s: %d' % (self._names[0], scores[0])
        blue_text = '%s: %d' % (self._names[1], scores[1])

        info = ''
        current_action = ''
        if self._highlighted is not None:
            if display_state.is_robot(self._highlighted):
                robot = display_state.robots[self._highlighted]
                info = 'Robot #%d' % robot.robot_id
                if not self.show_arrows.get():
                    bots_activity = self._game.get_actions_on_turn(
                        display_turn)
                    actioninfo = bots_activity[self._highlighted]
                    if actioninfo.get('name') is not None:
                        current_action += 'Current Action: {0}'.format(
                            actioninfo['name'])
                        if self._highlighted_target is not None:
                            current_action += (' to %s' %
                                               (self._highlighted_target,))
        # if self._highlighted is not None:
        #     if self._highlighted in settings.obstacles:
        #         info = 'Obstacle at '
        #     elif display_state.is_robot(self._highlighted):
        #         robot = display_state.robots[self._highlighted]
        #         info = 'Robot #%d at ' % robot.robot_id
        #     else:
        #         info = ''

        #     white_text.append(
        #         'Highlighted: %s%s' % (info, self._highlighted)
        #     )

        white_text = [
            'Turn: %d/%d' % (display_turn, settings.max_turns),
            'Highlighted: %s; %s' % (self._highlighted, info),
            current_action
        ]
        self._info.itemconfig(self._label, text='\n'.join(white_text))
        self._info.itemconfig(self._labelred, text=red_text)
        self._info.itemconfig(self._labelblue, text=blue_text)

    def current_turn_int(self):
        return min(int(math.floor(self._turn + self._sub_turn)),
                   settings.max_turns)

    def update_slider_value(self):
        v = -self._time_slider.get()
        if v > 0:
            v = v * 20
        self._slider_delay = render_settings.turn_interval + v

    def callback(self):
        self.update_slider_value()
        self.tick()
        self._win.after(1000 // render_settings.FPS, self.callback)

    def tick(self):
        now = millis()

        self.update_info_frame()
        # check if frame-update
        if self._animations:
            if not self._paused:
                self._sub_turn = max(0.0, (now - self._t_frame_start) /
                                     self._slider_delay)
                if self._turn >= settings.max_turns:
                    self.toggle_pause()
                    self._turn = settings.max_turns
                if self._sub_turn >= 1:
                    self._sub_turn -= 1
                    self._turn += 1
                    self.update_frame_start_time(self._t_next_frame)
                    self.turn_changed()
            subframe_t = ((now - self._t_cursor_start) %
                          render_settings.rate_cursor_blink)
            subframe_hlt = subframe_t / render_settings.rate_cursor_blink
            self.paint(self._sub_turn, subframe_hlt)
        elif now > self._t_next_frame and not self._paused:
            self._turn += 1
            self.update_frame_start_time(self._t_next_frame)
            self.turn_changed()
            self.paint()

        self.update_block_size()

    def get_bg_color(self, loc):
        if loc in settings.obstacles:
            return rgb_to_hex(*render_settings.obstacle_color)
        return rgb_to_hex(*render_settings.normal_color)

    def draw_background(self):
        # draw squares
        for y in range(settings.board_size):
            for x in range(settings.board_size):
                loc = (x, y)
                self.draw_grid_object(
                    loc, fill=self.get_bg_color(loc), layer=1, width=0)
        # draw text labels
        text_color = rgb_to_hex(*render_settings.text_color)
        for y in range(settings.board_size):
            self.draw_text((y, 0), str(y), color=text_color)
            self.draw_text((0, y), str(y), color=text_color)

    def update_sprites_new_turn(self):
        for sprite in self._sprites:
            sprite.clear()
        self._sprites = []

        self.update_highlight_sprite()
        turn_action = self.current_turn_int()
        bots_activity = self._game.get_actions_on_turn(turn_action)
        for bot_data in list(bots_activity.values()):
            self._sprites.append(RobotSprite(bot_data, self))

    def update_highlight_sprite(self, repaint=False):
        if self._highlight_sprite is not None:
            self._highlight_sprite.clear()
        self._highlight_sprite = HighlightSprite(
            self._highlighted, self._highlighted_target, self)
        if repaint:
            self.paint_highlight_sprite()

    def paint_highlight_sprite(self, subframe_hlt=0):
        if self._highlight_sprite is not None:
            self._highlight_sprite.animate(subframe_hlt)
            self.update_layers()

    def paint(self, subframe=0, subframe_hlt=0):
        for sprite in self._sprites:
            sprite.animate(subframe)
        self.update_highlight_sprite()
        self.paint_highlight_sprite(subframe_hlt)
        self.update_layers()

    def grid_to_xy(self, loc):
        x, y = loc
        return (x * self._blocksize + self.board_margin // 2,
                y * self._blocksize + self.board_margin // 2)

    def square_bottom_corner(self, square_topleft):
        x, y = square_topleft
        return (x + self._blocksize - self.cell_border_width,
                y + self._blocksize - self.cell_border_width)

    def grid_bbox(self, loc):
        x, y = self.grid_to_xy(loc)
        return (int(x),
                int(y),
                int(x + self._blocksize - self.cell_border_width),
                int(y + self._blocksize - self.cell_border_width))



import ast
import pkg_resources
import unittest

from rgkit.gamestate import GameState
from rgkit.settings import settings

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestStateMisc(unittest.TestCase):
    def test_add_robot(self):
        state = GameState()
        state.add_robot((9, 9), 0, robot_id=7, hp=42)
        self.assertTrue((9, 9) in state.robots)
        self.assertEqual(state.robots[(9, 9)].player_id, 0)
        self.assertEqual(state.robots[(9, 9)].robot_id, 7)
        self.assertEqual(state.robots[(9, 9)].hp, 42)

    def test_robot_ids_are_unique(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        state.add_robot((8, 8), 0)
        self.assertNotEqual(state.robots[(9, 9)].robot_id,
                             state.robots[(8, 8)].robot_id)

    def test_is_robot(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        self.assertTrue(state.is_robot((9, 9)))
        self.assertFalse(state.is_robot((8, 8)))

    def test_remove_robot(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        state.remove_robot((9, 9))
        self.assertFalse(state.is_robot((9, 9)))

    def test_scores(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        state.add_robot((6, 11), 1)
        state.add_robot((8, 14), 0)
        self.assertEqual(state.get_scores(), [2, 1])

    def test_get_game_info(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        state.add_robot((6, 11), 1)
        game_info = state.get_game_info(0)
        self.assertEqual(game_info.robots[9, 9].location, (9, 9))
        self.assertEqual(game_info.robots[9, 9].hp, state.robots[(9, 9)].hp)
        self.assertEqual(game_info.robots[9, 9].player_id,
                          state.robots[(9, 9)].player_id)
        self.assertEqual(game_info.robots[9, 9].robot_id,
                          state.robots[(9, 9)].robot_id)
        self.assertEqual(game_info.robots[6, 11].location, (6, 11))
        self.assertEqual(game_info.robots[6, 11].hp, state.robots[(6, 11)].hp)
        self.assertEqual(game_info.robots[6, 11].player_id,
                          state.robots[(6, 11)].player_id)
        self.assertRaises(AttributeError,
                          lambda: game_info.robots[6, 11].robot_id)
        self.assertEqual(game_info.turn, state.turn)




import unittest

import rgkit
from rgkit.run import Runner, Options
from rgkit.settings import settings as rgkit_settings
import rgkit.gamestate as gamestate


class TestRobot:
    def act(self, game):
        return ['guard']

called = False


class TestRunner(unittest.TestCase):
    def test_default_options(self):
        options = Options()
        assert options.map_filepath is not None
        assert isinstance(options.map_filepath, str)

        assert isinstance(options.print_info, bool)
        assert isinstance(options.n_of_games, int)
        assert isinstance(options.animate_render, bool)

    def test_default_settings(self):
        settings = Runner.default_settings()
        assert settings is not None
        assert settings == rgkit_settings

    def test_runner_from_robots(self):
        print((TestRobot().__class__))
        runner = Runner.from_robots([TestRobot(), TestRobot()])
        self.assertEqual(runner.options, Options())
        runner.settings.max_turns = 1
        runner.options.headless = True
        runner.options.quiet = 4
        runner.run()

    def test_runner_delta_callback(self):
        def callback(delta, game_state):
            global called
            called = True
            assert type(delta) is list
            assert type(delta[0]) is rgkit.settings.AttrDict
            assert type(game_state) is gamestate.GameState

        runner = Runner.from_robots([TestRobot(), TestRobot()],
                                    delta_callback=callback)
        runner.options.headless = True
        runner.options.quiet = 4
        runner.run()
        assert called

    def test_runner_from_robots_perf(self):

        runner = Runner.from_robots([TestRobot(), TestRobot()])
        runner.options.n_of_games = 100
        runner.settings.max_turns = 100
        # uncomment for profiling
        # import cProfile
        # cProfile.runctx("runner.runs()", globals(), locals())



import time
from rgkit.render.settings import settings as render_settings


def millis():
    return int(time.time() * 1000)


def rgb_to_hex(r, g, b, normalized=True):
    if normalized:
        # return '#%02x%02x%02x' % (r * 255, g * 255, b * 255)
        return '#%02x%02x%02x' % (int(r * 255), int(g * 255), int(b * 255))
    else:
        return '#%02x%02x%02x' % (r, g, b)


def rgb_tuple_to_hex(rgb, normalized=True):
    return rgb_to_hex(rgb[0], rgb[1], rgb[2], normalized)


def blend_colors(color1, color2, weight):
    r1, g1, b1 = color1
    r2, g2, b2 = color2
    r = r1 * weight + r2 * (1 - weight)
    g = g1 * weight + g2 * (1 - weight)
    b = b1 * weight + b2 * (1 - weight)
    return (r, g, b)


def compute_color(player_id, hp, action):
    r, g, b = render_settings.colors[player_id]
    maxclr = min(hp, 50)
    r += (100 - maxclr * 1.75) / 255
    g += (100 - maxclr * 1.75) / 255
    b += (100 - maxclr * 1.75) / 255
    color = (r, g, b)
    if action == 'guard' and render_settings.color_guard is not None:
        color = blend_colors(color, render_settings.color_guard, 0.65)
    return color



try:
    import tkinter
except ImportError:
    import tkinter as Tkinter
from rgkit.settings import settings
from rgkit.render.settings import settings as render_settings
from rgkit.render.utils import rgb_to_hex, rgb_tuple_to_hex, blend_colors
from rgkit.render.utils import compute_color


class RobotSprite(object):
    def __init__(self, action_info, render):
        self.location = action_info['loc']
        self.location_next = action_info['loc_end']
        self.action = action_info['name']
        self.target = action_info['target']
        self.hp = max(0, action_info['hp'])
        self.hp_next = max(0, action_info['hp_end'])
        self.id = action_info['player']
        self.renderer = render
        self.animation_offset = (0, 0)

        # Tkinter objects
        self.square = None
        self.border = None
        self.circle = None
        self.overlay = None
        self.text = None

    def animate(self, delta=0):
        """Animate this sprite.

           Delta is between 0 and 1. It tells us how far along to render (0.5
           is halfway through animation). This allows animation logic to be
           separate from timing logic.
        """
        # fix delta to between 0 and 1
        delta = max(0, min(delta, 1))
        bot_rgb_base = compute_color(self.id, self.hp,
                                     self.action)

        # default settings
        alpha_hack = 1
        bot_rgb = bot_rgb_base

        # if spawn, fade in
        normal_color = render_settings.normal_color
        if render_settings.bot_die_animation:
            if self.action == 'spawn':
                alpha_hack = delta
                bot_rgb = blend_colors(bot_rgb_base,
                                       normal_color, alpha_hack)
            # if dying, fade out
            elif self.hp_next <= 0:
                alpha_hack = 1 - delta
                bot_rgb = blend_colors(bot_rgb_base,
                                       normal_color, alpha_hack)

        x, y = self.location
        bot_size = self.renderer._blocksize
        self.animation_offset = (0, 0)
        arrow_fill = None

        # move animations
        if self.action == 'move' and self.target is not None:
            if (self.renderer._animations and
                    render_settings.bot_move_animation):
                # If normal move, start at bot location and move to next
                # location (note that first half of all move animations is the
                # same).
                if delta < 0.5 or self.location_next == self.target:
                    x, y = self.location
                    tx, ty = self.target
                # If we're halfway through this animation AND the movement
                # didn't succeed, reverse it (bounce back).
                else:
                    # starting where we wanted to go
                    x, y = self.target
                    # and ending where we are now
                    tx, ty = self.location
                dx = tx - x
                dy = ty - y
                off_x = dx * delta * self.renderer._blocksize
                off_y = dy * delta * self.renderer._blocksize
                self.animation_offset = (off_x, off_y)
            if render_settings.draw_movement_arrow:
                arrow_fill = 'lightblue'

        # attack animations
        elif self.action == 'attack' and self.target is not None:
            arrow_fill = 'orange'

        # guard animations
        elif self.action == 'guard':
            pass

        # suicide animations
        elif self.action == 'suicide':
            if (self.renderer._animations and
                    render_settings.bot_suicide_animation):
                # explosion animation
                # expand size (up to 1.5x original size)
                bot_size = self.renderer._blocksize * (1 + delta / 2)
                # color fade to yellow
                bot_rgb = blend_colors(bot_rgb, (1, 1, 0), 1 - delta)

        # DRAW ARROWS AND BORDER AND CIRCLE
        if self.renderer.show_arrows.get():
            if arrow_fill is not None and self.overlay is None:
                offset = (self.renderer._blocksize / 2,
                          self.renderer._blocksize / 2)
                self.overlay = self.renderer.draw_line(
                    self.location, self.target, layer=5, fill=arrow_fill,
                    offset=offset, width=3.0, arrow=tkinter.LAST)
            if self.action == 'guard' and self.border is None:
                self.border = self.renderer.draw_grid_object(
                    self.location, shape=render_settings.bot_shape, layer=4,
                    outline=rgb_tuple_to_hex(
                        render_settings.color_guard_border),
                    width=2)
            if self.action == 'suicide' and self.circle is None:
                self.circle = self.renderer.draw_grid_object(
                    self.location, shape="circle", layer=4,
                    outline="yellow",
                    width=2)
        else:
            if self.overlay is not None:
                self.renderer.remove_object(self.overlay)
                self.overlay = None
            if self.border is not None:
                self.renderer.remove_object(self.border)
                self.border = None
            if self.circle is not None:
                self.renderer.remove_object(self.circle)
                self.circle = None

        # DRAW BOTS WITH HP
        bot_hex = rgb_to_hex(*bot_rgb)
        self.draw_bot((x, y), bot_hex, bot_size)
        if render_settings.bot_hp_animation:
            self.draw_bot_hp(delta, (x, y), bot_rgb, alpha_hack)
        else:
            self.draw_bot_hp(0, (x, y), bot_rgb, alpha_hack)

    def draw_bot(self, loc, color, size):
        x, y, rx, ry = self.renderer.grid_bbox(loc)
        ox, oy = self.animation_offset
        if self.square is None:
            self.square = self.renderer.draw_grid_object(
                self.location, shape=render_settings.bot_shape, layer=1,
                fill=color, width=0)
        self.renderer._win.itemconfig(self.square, fill=color)
        self.renderer._win.coords(self.square,
                                  (x + ox, y + oy, rx + ox, ry + oy))

    def draw_bot_hp(self, delta, loc, bot_color, alpha):
        x, y = self.renderer.grid_to_xy(loc)
        ox, oy = self.animation_offset
        tex_rgb = render_settings.text_color_bright \
            if self.hp > settings.robot_hp / 4 \
            else render_settings.text_color_dark
        tex_rgb = blend_colors(tex_rgb, bot_color, alpha)
        tex_hex = rgb_to_hex(*tex_rgb)
        val = int(self.hp * (1 - delta) + self.hp_next * delta)
        if self.text is None:
            self.text = self.renderer.draw_text(self.location, val, tex_hex)
        self.renderer._win.itemconfig(self.text, text=val, fill=tex_hex)
        self.renderer._win.coords(
            self.text,
            (x + ox +
                (self.renderer._blocksize -
                    self.renderer.cell_border_width) / 2,
             y + oy +
                (self.renderer._blocksize -
                    self.renderer.cell_border_width) / 2))

    def clear(self):
        self.renderer.remove_object(self.square)
        self.renderer.remove_object(self.border)
        self.renderer.remove_object(self.circle)
        self.renderer.remove_object(self.overlay)
        self.renderer.remove_object(self.text)
        self.square = None
        self.border = None
        self.circle = None
        self.overlay = None
        self.text = None



class AttrDict(dict):
    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self


class Settings(AttrDict):
    def __init__(self, *args, **kwargs):
        super(Settings, self).__init__(*args, **kwargs)

    def init_map(self, map_data):
        self.spawn_coords = map_data['spawn']
        self.obstacles = map_data['obstacle']
        self.player_count = map_data.get('player_count', 2)
        self.start = map_data.get('start', None)


settings = Settings({
    'spawn_every': 10,
    'spawn_per_player': 5,
    'board_size': 19,
    'robot_hp': 50,
    'attack_range': (8, 10),
    'collision_damage': 5,
    'suicide_damage': 15,
    'max_turns': 100,
    'str_limit': 50,  # limit on length of representation of action
    'max_seed': 2147483647,

    # rating systems
    'default_rating': 1200,

    # user-scripting
    'max_time_initialization': 2000,
    'max_time_first_act': 1500,
    'max_time_per_act': 300,
    'exposed_properties': ('location', 'hp', 'player_id'),
    'player_only_properties': ('robot_id',),
    'user_obj_types': ('Robot',),
    'valid_commands': ('move', 'attack', 'guard', 'suicide')
})



from rgkit.settings import settings


CENTER_POINT = (int(settings.board_size / 2), int(settings.board_size / 2))


def dist(p1, p2):
    return ((p2[0] - p1[0]) ** 2 + (p2[1] - p1[1]) ** 2) ** 0.5


def wdist(p1, p2):
    return abs(p2[0] - p1[0]) + abs(p2[1] - p1[1])


def memoize(f):
    """ Memoization decorator for a function taking a single argument """
    class MemoDict(dict):
        def __missing__(self, key):
            ret = self[key] = f(key)
            return ret
    return MemoDict().__getitem__


def memodict(f):
    """Backward compatibility."""
    return memoize(f)


@memoize
def loc_types(loc):
    for i in range(2):
        if not 0 <= loc[i] < settings.board_size:
            return set(['invalid'])

    types = set(['normal'])
    if loc in settings.spawn_coords:
        types.add('spawn')
    if loc in settings.obstacles:
        types.add('obstacle')
    return types


@memoize
def _locs_around(loc):
    x, y = loc
    offsets = ((0, 1), (1, 0), (0, -1), (-1, 0))
    return [(x + dx, y + dy) for dx, dy in offsets]


def locs_around(loc, filter_out=None):
    filter_out = set(filter_out or [])
    return [a_loc for a_loc in _locs_around(loc)
            if len(filter_out & loc_types(a_loc)) == 0]


def _sign(x):
    return x and 1 if x > 0 else -1


def toward(curr, dest):
    if curr == dest:
        return curr

    x0, y0 = curr
    x, y = dest
    x_diff, y_diff = x - x0, y - y0

    move_y = (x0, y0 + _sign(y_diff))
    move_x = (x0 + _sign(x_diff), y0)

    if abs(y_diff) > abs(x_diff):
        if move_y not in settings.obstacles:
            return move_y
        else:
            return move_x
    else:
        if move_x not in settings.obstacles:
            return move_x
        else:
            return move_y



#!/usr/bin/env python2


import argparse
from argparse import RawTextHelpFormatter
import ast
import copy
from importlib import import_module
import inspect
import pkg_resources
import random
import os
import sys
import time

try:
    import_module('rgkit')
except ImportError:
    # force rgkit to appear as a module when run from current directory
    from os.path import dirname, abspath
    cdir = dirname(abspath(inspect.getfile(inspect.currentframe())))
    parentdir = dirname(cdir)
    sys.path.insert(0, parentdir)

from rgkit.settings import settings as default_settings
from rgkit import game
from rgkit.game import Player


class Options(object):
    def __init__(self, map_filepath=None, headless=False, print_info=False,
                 animate_render=False, play_in_thread=False, curses=False,
                 game_seed=None, match_seeds=None, quiet=0, symmetric=True,
                 n_of_games=1, start=0):

        if map_filepath is None:
            map_filepath = os.path.join(os.path.dirname(__file__),
                                        'maps/default.py')
        self.animate_render = animate_render
        self.curses = curses
        self.game_seed = game_seed
        self.headless = headless
        self.map_filepath = map_filepath
        self.match_seeds = match_seeds
        self.n_of_games = n_of_games
        self.play_in_thread = play_in_thread
        self.print_info = print_info
        self.quiet = quiet
        self.start = start
        self.symmetric = symmetric

    def __eq__(self, other):
        return (self.animate_render == other.animate_render and
                self.curses == other.curses and
                self.game_seed == other.game_seed and
                self.headless == other.headless and
                self.map_filepath == other.map_filepath and
                self.match_seeds == other.match_seeds and
                self.n_of_games == other.n_of_games and
                self.play_in_thread == other.play_in_thread and
                self.print_info == other.print_info and
                self.quiet == other.quiet and
                self.start == other.start and
                self.symmetric == other.symmetric)


class Runner(object):
    def __init__(self, players=None, player_files=None, settings=None,
                 options=None, delta_callback=None):

        if settings is None:
            settings = Runner.default_settings()
        if options is None:
            options = Options()
        if players is None:
            players = []

        self._map_data = ast.literal_eval(open(options.map_filepath).read())
        self.settings = settings
        self.settings.init_map(self._map_data)
        # Players can only be initialized from file after initializing settings
        if player_files is not None:
            for player_file in player_files:
                players.append(self._make_player(player_file))
        self._players = players
        self._delta_callback = delta_callback
        self._names = []
        for player in players:
            self._names.append(player.name())
        self.options = options

        if Runner.is_multiprocessing_supported():
            import multiprocessing
            self._rgcurses_lock = multiprocessing.Lock()
        else:
            self._rgcurses_lock = None

    @staticmethod
    def from_robots(robots, settings=None, options=None,
                    delta_callback=None):

        players = []
        for robot in robots:
            players.append(Player(robot=robot))

        return Runner(players,
                      settings=settings, options=options,
                      delta_callback=delta_callback)

    @staticmethod
    def from_command_line_args(args):
        map_name = os.path.join(args.map)

        options = Options(animate_render=args.animate,
                          curses=args.curses,
                          game_seed=args.game_seed,
                          headless=args.headless,
                          map_filepath=map_name,
                          match_seeds=args.match_seeds,
                          n_of_games=args.count,
                          play_in_thread=args.play_in_thread,
                          print_info=not args.headless and args.quiet <= 2,
                          quiet=args.quiet,
                          start=args.start,
                          symmetric=not args.random)
        # TODO: generalize to N player files
        player_files = [args.player1, args.player2]
        return Runner(player_files=player_files, options=options)

    @staticmethod
    def _make_player(file_name):
        try:
            return game.Player(file_name=file_name)
        except IOError as msg:
            if pkg_resources.resource_exists('rgkit', file_name):
                bot_filename = pkg_resources.resource_filename('rgkit',
                                                               file_name)
                return game.Player(file_name=bot_filename)
            raise IOError(msg)

    @staticmethod
    def default_map():
        map_path = os.path.join(os.path.dirname(__file__), 'maps/default.py')
        return map_path

    @staticmethod
    def default_settings():
        return default_settings

    def game(self, record_turns=False, unit_testing=False):
        return game.Game(self._players, record_turns=record_turns,
                         unit_testing=unit_testing)

    def run(self):
        scores = []
        printed = []
        for i in range(self.options.start,
                       self.options.start + self.options.n_of_games):
            # A sequential, deterministic seed is used for each match that can
            # be overridden by user provided ones.
            match_seed = str(self.options.game_seed) + '-' + str(i)
            if self.options.match_seeds and i < len(self.options.match_seeds):
                match_seed = self.options.match_seeds[i]
            for player in self._players:
                player.load()
            result = self.play(match_seed)
            scores.append(result)
            printed.append('{0} - seed: {1}'.format(result, match_seed))
        if self.options.quiet < 4:
            unmute_all()
            if printed:
                print('\n'.join(printed))
        return scores

    def play(self, match_seed):
        if self.options.play_in_thread:
            g = game.ThreadedGame(self._players,
                                  print_info=self.options.print_info,
                                  record_actions=not self.options.headless,
                                  record_history=True,
                                  seed=match_seed,
                                  quiet=self.options.quiet,
                                  delta_callback=self._delta_callback,
                                  symmetric=self.options.symmetric)
        else:
            g = game.Game(self._players,
                          print_info=self.options.print_info,
                          record_actions=not self.options.headless,
                          record_history=True,
                          seed=match_seed,
                          quiet=self.options.quiet,
                          delta_callback=self._delta_callback,
                          symmetric=self.options.symmetric)

        if not self.options.headless and not self.options.curses:
            # only import render if we need to render the game;
            # this way, people who don't have tkinter can still
            # run headless
            from rgkit.render import render

        g.run_all_turns()

        if not self.options.headless and not self.options.curses:
            # print "rendering %s animations" % ("with"
            #                                    if animate_render
            #                                    else "without")
            render.Render(g, self.options.animate_render, names=self._names)

        # TODO: Displaying multiple games using curses is still a little bit
        # buggy but at least it doesn't completely screw up the state of the
        # terminal anymore.  The plan is to show each game sequentially.
        # Concurrency in run.py needs some more work before the bugs can be
        # fixed. Need to make sure nothing is printing when curses is running.
        if not self.options.headless and self.options.curses:
            from rgkit import rgcurses
            rgc = rgcurses.RGCurses(g, self._names)
            if self._rgcurses_lock:
                self._rgcurses_lock.acquire()
            rgc.run()
            if self._rgcurses_lock:
                self._rgcurses_lock.release()

        return g.get_scores()

    @staticmethod
    def is_multiprocessing_supported():
        is_multiprocessing_supported = True
        try:
            import_module('multiprocessing')
        except ImportError:
            # the OS does not support it. See http://bugs.python.org/issue3770
            is_multiprocessing_supported = False

        return is_multiprocessing_supported


def run_single_from_command_line(args):
    return Runner.from_command_line_args(args).run()


def run_concurrently(args):
    import multiprocessing
    num_cpu = multiprocessing.cpu_count()
    (games_per_cpu, remainder) = divmod(args.count, num_cpu)
    data = []
    start = 0

    for _ in range(num_cpu):
        copy_args = copy.deepcopy(args)

        copy_args.start = start
        copy_args.count = games_per_cpu
        start += games_per_cpu
        # Distribute remainder of games evenly among CPUs
        if remainder > 0:
            copy_args.count += 1
            start += 1
            remainder -= 1

        data.append(copy_args)

    pool = multiprocessing.Pool(num_cpu)
    results = pool.map(run_single_from_command_line, data)
    return [score for scores in results for score in scores]


def get_arg_parser():
    parser = argparse.ArgumentParser(
        description="Robot game execution script.",
        formatter_class=RawTextHelpFormatter)
    parser.add_argument("player1",
                        help="File containing first robot class definition.")
    parser.add_argument("opponents", nargs="+",
                        help="File(s) containing opponent robots.")
    default_map = pkg_resources.resource_filename('rgkit', 'maps/default.py')
    parser.add_argument("-m", "--map",
                        help="User-specified map file.",
                        default=default_map)
    parser.add_argument("-c", "--count", type=int,
                        default=1,
                        help="Game count, default: 1, multithreading if >1")
    parser.add_argument("-A", "--animate", action="store_true",
                        default=False,
                        help="Enable animations in rendering.")
    parser.add_argument(
        "-q", "--quiet", action="count", default=0, help="""Quiet execution.
-q : suppresses bot stdout
-qq: suppresses bot stdout and stderr
-qqq: supresses all rgkit and bot output
-qqqq: final summary only""")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-H", "--headless", action="store_true",
                       default=False,
                       help="Disable rendering game output.")
    group.add_argument("-T", "--play-in-thread", action="store_true",
                       default=False,
                       help="Separate GUI thread from robot move calculations."
                       )
    group.add_argument("-C", "--curses", action="store_true",
                       default=False,
                       help="Display game in command line using curses.")
    parser.add_argument("--game-seed",
                        default=random.randint(0, default_settings.max_seed),
                        help="Appended with game countfor per-match seeds.")
    parser.add_argument(
        "--match-seeds", nargs='*',
        help="Used for random seed of the first matches in order.")
    parser.add_argument("-r", "--random", action="store_true",
                        default=False,
                        help="Bots spawn randomly instead of symmetrically.")
    parser.add_argument("-M", "--heatmap", action="store_true",
                        default=False,
                        help="Print heatmap after playing a number of games.")
    parser.add_argument("-s", "--start", type=int, default=0,
                        help="Starting index of matches, useful for resuming.")
    try:
        os.nice(0)
        parser.add_argument("--nice", type=int, default=5,
                            help="Value for os.nice to lower runner priority.")
    except:
        # Not available on this platform, no need to add the option.
        pass

    return parser


def mute_all():
    sys.stdout = game.NullDevice()
    # sys.stderr = game.NullDevice()


def unmute_all():
    sys.stdout = sys.__stdout__
    # sys.stderr = sys.__stderr__


def print_score_grid(scores, player1, player2, size):
    max_score = 50

    def to_grid(n):
        return int(round(float(n) / max_score * (size - 1)))

    def print_heat(n):
        if n > 9:
            sys.stdout.write(" +")
        else:
            sys.stdout.write(" " + str(n))

    grid = [[0 for c in range(size)] for r in range(size)]

    for s1, s2 in scores:
        grid[to_grid(s1)][to_grid(s2)] += 1

    p1won = sum(p1 > p2 for p1, p2 in scores)
    str1 = player1 + " : " + str(p1won)
    if len(str1) + 2 <= 2 * size - len(str1):
        str1 = " " + str1 + " "
        print("*" + str1 + "-" * (2 * size - len(str1)) + "*")
    else:
        print(str1)
        print("*" + "-" * (2 * size) + "*")

    for r in range(size - 1, -1, -1):
        sys.stdout.write("|")
        for c in range(size):
            if grid[r][c] == 0:
                if r == c:
                    sys.stdout.write(". ")
                else:
                    sys.stdout.write("  ")
            else:
                print_heat(grid[r][c])
        sys.stdout.write("|\n")

    p2won = sum(p2 > p1 for p1, p2 in scores)
    str2 = player2 + " : " + str(p2won)
    if len(str2) + 2 <= 2 * size - len(str2):
        str2 = " " + str2 + " "
        print("*" + "-" * (2 * size - len(str2)) + str2 + "*")
    else:
        print("*" + "-" * (2 * size) + "*")
        print(str2)


def main():
    args = get_arg_parser().parse_args()

    if "nice" in args:
        os.nice(args.nice)

    num_opponents = len(args.opponents)
    total_won, total_lost, total_draw, total_avg_score, total_diff = (
        0, 0, 0, (0, 0), 0)
    for opponent in args.opponents:
        args.player2 = opponent
        if args.quiet >= 3:
            mute_all()
        print('Game seed: {0}'.format(args.game_seed))
        if Runner.is_multiprocessing_supported() and args.count > 1:
            runner = run_concurrently
        else:
            runner = run_single_from_command_line

        start_time = time.time()
        scores = runner(args)
        total_time = time.time() - start_time

        print('{0:6.2f}s per game, {1} games, total {2:.0f}s'.format(
            total_time / args.count, args.count, total_time))
        if args.quiet >= 3:
            unmute_all()
        p1won = sum(p1 > p2 for p1, p2 in scores)
        p2won = sum(p2 > p1 for p1, p2 in scores)
        draw = args.count - p1won - p2won
        avg_score = [float(sum(x))/len(x) for x in zip(*scores)]
        diff = avg_score[0] - avg_score[1]
        if args.heatmap:
            print_score_grid(scores, args.player1, args.player2, 26)
        total_won += p1won
        total_lost += p2won
        total_draw += draw
        total_avg_score = (total_avg_score[0] + avg_score[0],
                           total_avg_score[1] + avg_score[1])
        total_diff += diff
        avg_score = list(map(int, avg_score))
        diff = int(diff)
        print('{:10} - {:15} - {:8} ({})'.format(
            os.path.basename(opponent)[:10], repr([p1won, p2won, draw]),
            repr(avg_score), diff))

    if num_opponents > 1:
        total_avg_score = list(map(int, (total_avg_score[0] / num_opponents,
                                         total_avg_score[1] / num_opponents)))
        total_diff = int(total_diff / num_opponents)
        win_rate = (100 * float(total_won + 0.5 * total_draw) /
                    (total_won + total_lost + total_draw))
        print('Overall: [{}, {}, {}] WR: {:<5.1f} Score: {} ({})'.format(
            total_won, total_lost, total_draw, win_rate, total_avg_score,
            total_diff))


if __name__ == '__main__':
    main()



from rgkit.settings import AttrDict


settings = AttrDict({
    'FPS': 60,  # frames per second
    'turn_interval': 300,  # milliseconds per turn

    # colors
    'colors': [(0.49, 0.14, 0.14), (0.14, 0.14, 0.49)],
    'color_guard': None,  # (0.0, 0.14, 0.0),
    'color_guard_border': (0.0, 0.49, 0.0),
    # 'colors': [(0.9, 0, 0.2), (0, 0.9, 0.2)],
    'obstacle_color': (.2, .2, .2),
    'text_color': (0.6, 0.6, 0.6),  # for labelling rows/columns
    'text_color_dark': (0.1, 0.1, 0.1),  # HP color when bots are bright
    'text_color_bright': (0.9, 0.9, 0.9),  # HP color when bots are dark
    'normal_color': (.9, .9, .9),
    'highlight_color': (0.6, 0.6, 0.6),
    'target_color': (0.6, 0.6, 1),

    # highlighting
    'clear_highlight_between_turns': False,
    # 'clear_highlight_between_turns': True,
    'clear_highlight_target_between_turns': True,
    'highlight_cursor_blink': True,
    'rate_cursor_blink': 1000,
    # 'highlight_cursor_blink': True,
    'highlight_cursor_blink_interval': 0.5,

    'bot_shape': 'square',
    # 'bot_shape': 'circle',
    'draw_movement_arrow': True,
    # 'draw_movement_arrow': False,

    # animations (only enabled if -A is used)
    'bot_die_animation': True,
    'bot_move_animation': False,
    'bot_suicide_animation': False,
    'bot_hp_animation': False,
})



import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit import game

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestSeeding(unittest.TestCase):
    def test_seeding_saves_spawning(self):
        seed = 42
        file1 = pkg_resources.resource_filename(
            'rgkit', 'bots/guardbot.py')
        file2 = pkg_resources.resource_filename(
            'rgkit', 'bots/guardbot.py')

        player1 = game.Player(file1)
        player2 = game.Player(file2)
        game1 = game.Game([player1, player2],
                          print_info=False,
                          record_actions=False,
                          record_history=True,
                          seed=seed)

        game1.run_all_turns()

        player1 = game.Player(file1)
        player2 = game.Player(file2)
        game2 = game.Game([player1, player2],
                          print_info=False,
                          record_actions=True,
                          record_history=True,
                          seed=seed)

        game2.run_all_turns()

        self.assertEqual(game1.history, game2.history)

    def test_seeding_saves_random(self):
        seed = 42
        file1 = pkg_resources.resource_filename(
            'rgkit', 'bots/randombot.py')
        file2 = pkg_resources.resource_filename(
            'rgkit', 'bots/randombot.py')

        player1 = game.Player(file1)
        player2 = game.Player(file2)
        game1 = game.Game([player1, player2],
                          print_info=False,
                          record_actions=False,
                          record_history=True,
                          seed=seed)

        game1.run_all_turns()

        player1 = game.Player(file1)
        player2 = game.Player(file2)
        game2 = game.Game([player1, player2],
                          print_info=False,
                          record_actions=True,
                          record_history=True,
                          seed=seed)

        game2.run_all_turns()

        self.assertEqual(game1.history, game2.history)



import types
import os
import random
import sys
import traceback
import io

try:
    import threading
except ImportError:
    import dummy_threading as threading


from rgkit import rg
from rgkit.gamestate import GameState
from rgkit.settings import settings

sys.modules['rg'] = rg  # preserve backwards compatible robot imports


class NullDevice(object):
    def write(self, msg):
        pass

    def flush(self):
        pass


# TODO: use actual logging module with log levels and copying stdout
#       instead of this
class Tee(object):
    def __init__(self, orig, copy, conv=lambda s: s):
        self.orig = orig
        self.copy = copy
        self.conv = conv

    def write(self, message):
        self.orig.write(message)
        self.copy.write(self.conv(message))

    def flush(self):
        self.orig.flush()
        self.copy.flush()


class Player(object):
    def __init__(self, file_name=None, robot=None, code=None, name=None):
        """
        One of these arguments must be provided:
        file_name -- path to file containing a robot
        robot -- instance of a robot
        code -- source code containing a robot
        name argument can be used to set robot's name
        """
        self._player_id = None  # must be set using set_player_id

        self._code = code
        if file_name is not None:
            with open(file_name) as f:
                self._code = f.read()
            self._name = os.path.splitext(
                os.path.basename(file_name))[0]
        if name is not None:
            self._name = name
        self.load(robot)

    def load(self, robot=None):
        if robot is not None:
            self._name = str(robot.__class__).split('.')[-1]
            self._robot = robot
        elif self._code:
            self._module = types.ModuleType('usercode%d' % id(self))
            exec(self._code, self._module.__dict__)
            self._robot = self._module.Robot()
        else:
            # No way to reload robot...
            pass

    def set_player_id(self, player_id):
        self._player_id = player_id

    @staticmethod
    def _numeral_types():
        if sys.version_info >= (3, 0):
            return (int, float)
        else:
            return (int, int, float)  # noqa

    @staticmethod
    def _validate_type(robot, var_name, obj, types):
        if type(obj) not in types:
            raise Exception(
                "Bot {0}: {1} of type {2} is not valid.".format(
                    robot.robot_id, var_name, type(obj).__name__)
            )

    @staticmethod
    def _validate_length(robot, var_name, obj, lengths):
        if len(obj) not in lengths:
            # assumes that type of obj has already been validated and so
            # __repr__ has not been overwritten
            raise Exception(
                "Bot {0}: {1} of length {2} is not valid.".format(
                    robot.robot_id, var_name, len(obj))
            )

    @staticmethod
    def _validate_action(robot, action):
        """
        Need to be VERY CAREFUL here not to call any built-in functions on
        'action' unless it is known to be completely safe. A malicious bot may
        return an object with overwritten built-in functions that run arbitrary
        code.
        """
        Player._validate_type(robot, 'action', action, (list, tuple))
        Player._validate_length(robot, 'action', action, (1, 2))
        Player._validate_type(robot, 'action[0]', action[0], (str,))
        if action[0] in ('move', 'attack'):
            if len(action) != 2:
                raise Exception(
                    'Bot {0}: {1} requires a location as well.'.format(
                        robot.robot_id, action)
                )
            Player._validate_type(robot, 'action[1]', action[1], (list, tuple))
            Player._validate_length(robot, 'action[1]', action[1], (2,))
            Player._validate_type(
                robot, 'action[1][0]', action[1][0], Player._numeral_types())
            Player._validate_type(
                robot, 'action[1][1]', action[1][1], Player._numeral_types())
            valid_locs = rg.locs_around(
                robot.location, filter_out=['invalid', 'obstacle'])
            if action[1] not in valid_locs:
                raise Exception(
                    'Bot {0}: {1} is not a valid action.'.format(
                        robot.robot_id, action)
                )
        elif action[0] not in ('guard', 'suicide'):
            raise ValueError('Bot %d: action must be one of "guard", '
                             '"suicide", "move", or "attack".')

    def _get_response(self, game_state, game_info, robot, seed):
        """Returns sanitized action, output and error flag from robot"""
        try:
            exc_flag = False
            captured_output = io.BytesIO()

            def conv(s):
                return s.encode('ascii', 'replace')

            _stdout = sys.stdout
            _stderr = sys.stderr
            sys.stdout = Tee(sys.stdout, captured_output, conv=conv)
            sys.stderr = Tee(sys.stderr, captured_output, conv=conv)

            random.seed(seed)
            # Server requires knowledge of seed
            game_info.seed = seed

            self._robot.location = robot.location
            self._robot.hp = robot.hp
            self._robot.player_id = robot.player_id
            self._robot.robot_id = robot.robot_id
            action = self._robot.act(game_info)

            Player._validate_action(robot, action)

            if action[0] in ('move', 'attack'):
                action = (
                    action[0],
                    (int(action[1][0]), int(action[1][1]))
                )
            elif action[0] in ('guard', 'suicide'):
                action = (action[0],)

        except:
            exc_flag = True
            traceback.print_exc(file=sys.stderr)
            action = ('guard',)

        finally:
            sys.stdout = _stdout
            sys.stderr = _stderr

        return action, (exc_flag, captured_output.getvalue())

    def get_responses(self, game_state, seed):
        """
        Returns a tuple of two dictionaries containing actions,
        and (error flag and output) for each bot, respectively
        """
        game_info = game_state.get_game_info(self._player_id)
        actions, outputs = {}, {}

        for loc, robot in list(game_state.robots.items()):
            if robot.player_id == self._player_id:
                # Every act call should get a different random seed
                actions[loc], outputs[loc] = self._get_response(
                    game_state, game_info, robot,
                    seed=str(seed) + '-' + str(robot.robot_id))

        return actions, outputs

    def name(self):
        return self._name


class Game(object):
    def __init__(self, players, record_actions=False, record_history=False,
                 print_info=False, seed=None, quiet=0, delta_callback=None,
                 symmetric=True):
        self._players = players
        for i, player in enumerate(self._players):
            player.set_player_id(i)
        self._record_actions = record_actions
        self._record_history = record_history
        self._print_info = print_info
        if seed is None:
            seed = random.randint(0, settings.max_seed)
        self.seed = str(seed)
        self._random = random.Random(self.seed)
        self._quiet = quiet
        self._delta_callback = delta_callback
        self._state = GameState(use_start=True, seed=self.seed,
                                symmetric=symmetric)

        self._actions_on_turn = {}
        self._states = {}
        self.history = []  # TODO: make private

    # actions_on_turn = {loc: log_item}
    # log_item = {
    #     'name': action_name,
    #     'target': action_target or None,
    #     'loc': loc,
    #     'hp': hp,
    #     'player': player_id,
    #     'loc_end': loc_end,
    #     'hp_end': hp_end
    # }
    #
    # or dummy if turn == settings.max_turn
    def get_actions_on_turn(self, turn):
        assert self._record_actions
        return self._actions_on_turn[turn]

    def get_state(self, turn):
        return self._states[turn]

    def _save_actions_on_turn(self, actions_on_turn, turn):
        self._actions_on_turn[turn] = actions_on_turn

    def _save_state(self, state, turn):
        self._states[turn] = state

    def _get_robots_responses(self):
        # TODO: honour quietness
        actions, outputs = {}, {}
        for player in self._players:
            seed = self._random.randint(0, settings.max_seed)
            responses = player.get_responses(self._state, seed)
            actions.update(responses[0])
            outputs.update(responses[1])

        return actions, outputs

    def _make_history(self, responses, record_output=False):
        # todo: rework this. We are getting data about the player
        #       from two sources: 1) from arguments, and 2) from
        #       class members. Either move *all* to 1) and make
        #       static or move all to 2).
        '''
        An aggregate of all bots and their actions this turn.

        Optionally records per-bot output.

        Stores a list of each player's bots at the start of this turn and
        the actions they each performed this turn. Newly spawned bots have no
        actions.
        '''
        actions, outputs = responses
        robots = []
        for loc, robot in list(self._state.robots.items()):
            robot_info = {
                'location': loc,
                'hp': robot.hp,
                'player_id': robot.player_id,
                'robot_id': robot.robot_id,
            }
            if loc in actions:
                # since the state after the final turn does not contain any
                # actions, 'loc' is not always contained in 'actions'
                robot_info['action'] = actions[loc]
            if outputs and loc in outputs:
                robot_info['output'] = outputs[loc]
            robots.append(robot_info)
        return robots

    def _calculate_actions_on_turn(self, delta, actions):
        actions_on_turn = {}

        for delta_info in delta:
            loc = delta_info.loc

            if loc in actions:
                name = actions[loc][0]
                if name in ['move', 'attack']:
                    target = actions[loc][1]
                else:
                    target = None
            else:
                name = 'spawn'
                target = None

            # note that a spawned bot may overwrite an existing bot
            actions_on_turn[loc] = {
                'name': name,
                'target': target,
                'loc': loc,
                'hp': delta_info.hp,
                'player': delta_info.player_id,
                'loc_end': delta_info.loc_end,
                'hp_end': delta_info.hp_end
            }

        return actions_on_turn

    def run_turn(self, record_output=False):
        if self._print_info:
            print(((' running turn %d ' % (self._state.turn)).center(70, '-')))

        responses = self._get_robots_responses()
        actions = responses[0]

        delta = self._state.get_delta(actions)

        if self._record_actions:
            actions_on_turn = self._calculate_actions_on_turn(delta, actions)
            self._save_actions_on_turn(actions_on_turn, self._state.turn)

        new_state = self._state.apply_delta(delta)

        if self._delta_callback is not None and self._state.turn > 1:
            self._delta_callback(delta, new_state)

        self._save_state(new_state, new_state.turn)

        if self._record_history:
            self.history.append(self._make_history(
                responses, record_output=record_output))

        self._state = new_state

    def run_all_turns(self):
        assert self._state.turn == 0

        if self._print_info:
            print(('Match seed: {0}'.format(self.seed)))

        self._save_state(self._state, 0)

        while self._state.turn < settings.max_turns:
            self.run_turn()

        # create last turn's state for server history
        if self._record_history:
            self.history.append(self._make_history(({}, {})))

        # create dummy data for last turn
        # TODO: render should be cleverer
        actions_on_turn = {}

        for loc, robot in list(self._state.robots.items()):
            log_item = {
                'name': '',
                'target': None,
                'loc': loc,
                'hp': robot.hp,
                'player': robot.player_id,
                'loc_end': loc,
                'hp_end': robot.hp
            }

            actions_on_turn[loc] = log_item

        self._save_actions_on_turn(actions_on_turn, settings.max_turns)

    def get_scores(self):
        return self.get_state(settings.max_turns).get_scores()


class ThreadedGame(Game):
    def __init__(self, *args, **kwargs):
        super(ThreadedGame, self).__init__(*args, **kwargs)

        # events set when actions_on_turn are calculated
        self._has_actions_on_turn = [threading.Event()
                                     for _ in range(settings.max_turns + 1)]

        # events set when state are calculated
        self._has_state = [threading.Event()
                           for _ in range(settings.max_turns + 1)]

    def get_actions_on_turn(self, turn):
        self._has_actions_on_turn[turn].wait()
        return super(ThreadedGame, self).get_actions_on_turn(turn)

    def get_state(self, turn):
        self._has_state[turn].wait()
        return super(ThreadedGame, self).get_state(turn)

    def _save_actions_on_turn(self, actions_on_turn, turn):
        super(ThreadedGame, self)._save_actions_on_turn(actions_on_turn, turn)
        self._has_actions_on_turn[turn].set()

    def _save_state(self, state, turn):
        super(ThreadedGame, self)._save_state(state, turn)
        self._has_state[turn].set()

    def run_all_turns(self):
        lock = threading.Lock()

        def task():
            with lock:
                super(ThreadedGame, self).run_all_turns()

        turn_runner = threading.Thread(target=task)
        turn_runner.daemon = True
        turn_runner.start()



import curses as cs
from rgkit.settings import settings


class RGCurses(object):
    def __init__(self, game_inst, names):
        self._game = game_inst
        self._names = names
        self._turn = 0
        self._done = False
        self._paused = False
        self._selected = [settings.board_size // 2, settings.board_size // 2]

        # *** Edit settings below this line ***

        self._manual_text = """Exit            q, <esc>
Play/Pause      p, <space>
Next turn       j
Prev turn       k
Rewind turns    r
Cursor up       w, <up>
Cursor left     a, <left>
Cursor down     s, <down>
Cursor right    d, <right>"""

        self._turn_delay = 250

        # Positions
        self._game_grid_pos = (1, 2)
        self._score_pos = (19, 41)
        self._final_score_pos = (1, 41)
        self._manual_pos = (3, 41)
        self._cell_info_pos = (13, 41)
        self._turn_pos = (18, 41)

        # Keybindings
        self._exit_keys = [ord('q'), 27]
        self._pause_keys = [ord('p'), ord(' ')]
        self._step_keys = [ord('j')]
        self._back_keys = [ord('k')]
        self._rewind_keys = [ord('r')]
        self._up_keys = [ord('w'), cs.KEY_UP]
        self._left_keys = [ord('a'), cs.KEY_LEFT]
        self._down_keys = [ord('s'), cs.KEY_DOWN]
        self._right_keys = [ord('d'), cs.KEY_RIGHT]

    def _init_curses(self):

        # Colors and attributes

        colors_empty = 1
        colors_obstacle = 2
        colors_bot1 = 3
        colors_bot2 = 4
        # Selected
        colors_empty_s = 5
        colors_obstacle_s = 6
        colors_bot1_s = 7
        colors_bot2_s = 8
        # Other
        colors_text = 9

        # (Color pair, Foreground, Background)
        cs.init_pair(colors_empty, cs.COLOR_WHITE, cs.COLOR_BLACK)
        cs.init_pair(colors_obstacle, cs.COLOR_BLACK, cs.COLOR_WHITE)
        cs.init_pair(colors_bot1, cs.COLOR_WHITE, cs.COLOR_RED)
        cs.init_pair(colors_bot2, cs.COLOR_WHITE, cs.COLOR_BLUE)
        cs.init_pair(colors_empty_s, cs.COLOR_WHITE, cs.COLOR_YELLOW)
        cs.init_pair(colors_obstacle_s, cs.COLOR_BLACK, cs.COLOR_YELLOW)
        cs.init_pair(colors_bot1_s, cs.COLOR_WHITE, cs.COLOR_MAGENTA)
        cs.init_pair(colors_bot2_s, cs.COLOR_WHITE, cs.COLOR_CYAN)
        cs.init_pair(colors_text, cs.COLOR_WHITE, cs.COLOR_BLACK)

        # Attributes
        attr_empty = cs.A_NORMAL
        attr_obstacle = cs.A_NORMAL
        attr_bot1 = cs.A_BOLD
        attr_bot2 = cs.A_BOLD
        attr_empty_s = cs.A_NORMAL
        attr_obstacle_s = cs.A_NORMAL
        attr_bot1_s = cs.A_BOLD
        attr_bot2_s = cs.A_BOLD
        attr_text = cs.A_NORMAL

        # **** Do not edit settings below this line ***

        cs.curs_set(0)
        self._attr_empty = cs.color_pair(colors_empty) | attr_empty
        self._attr_obstacle = cs.color_pair(colors_obstacle) | attr_obstacle
        self._attr_bot1 = cs.color_pair(colors_bot1) | attr_bot1
        self._attr_bot2 = cs.color_pair(colors_bot2) | attr_bot2
        self._attr_empty_s = cs.color_pair(colors_empty_s) | attr_empty_s
        self._attr_obstacle_s = cs.color_pair(colors_obstacle_s) \
            | attr_obstacle_s
        self._attr_bot1_s = cs.color_pair(colors_bot1_s) | attr_bot1_s
        self._attr_bot2_s = cs.color_pair(colors_bot2_s) | attr_bot2_s
        self._attr_text = cs.color_pair(colors_text) | attr_text

    def run(self):
        # TODO: handle off-screen draws gracefully, instead of try-catch
        try:
            cs.wrapper(self._main)
        except:
            pass

    def _grid_num_to_str(self, n):
        if -9 <= n <= -1:
            return str(n)
        elif 0 <= n <= 9:
            return ' ' + str(n)
        elif 10 <= n <= 99:
            return str(n)
        else:
            return 'EE'     # Error! number too big for one cell

    def _draw_grid_empty(self, r, c):
        if [r, c] == self._selected:
            attr = self._attr_empty_s
        else:
            attr = self._attr_empty
        r = self._game_grid_pos[0] + 2 * r
        c = self._game_grid_pos[1] + c
        self._stdscr.addstr(c, r, '  ', attr)

    def _draw_grid_obstacle(self, r, c):
        if [r, c] == self._selected:
            attr = self._attr_obstacle_s
        else:
            attr = self._attr_obstacle
        if r == 0 and c % 2 == 0:
            show_str = self._grid_num_to_str(c)
        elif c == 0 and r % 2 == 0:
            show_str = self._grid_num_to_str(r)
        else:
            show_str = '  '
        r = self._game_grid_pos[0] + 2 * r
        c = self._game_grid_pos[1] + c
        self._stdscr.addstr(c, r, show_str, attr)

    def _draw_grid_bot1(self, r, c, hp):
        if [r, c] == self._selected:
            attr = self._attr_bot1_s
        else:
            attr = self._attr_bot1
        r = self._game_grid_pos[0] + 2 * r
        c = self._game_grid_pos[1] + c
        show_str = self._grid_num_to_str(hp)
        self._stdscr.addstr(c, r, show_str, attr)

    def _draw_grid_bot2(self, r, c, hp):
        if [r, c] == self._selected:
            attr = self._attr_bot2_s
        else:
            attr = self._attr_bot2
        r = self._game_grid_pos[0] + 2 * r
        c = self._game_grid_pos[1] + c
        show_str = self._grid_num_to_str(hp)
        self._stdscr.addstr(c, r, show_str, attr)

    def _draw_game_grid(self):
        state = self._game.get_state(self._turn)
        for r in range(settings.board_size):
            for c in range(settings.board_size):
                if (r, c) in settings.obstacles:
                    self._draw_grid_obstacle(r, c)
                elif state.is_robot((r, c)):
                    robot = state.robots[(r, c)]
                    if robot.player_id == 0:
                        self._draw_grid_bot1(r, c, robot.hp)
                    else:
                        self._draw_grid_bot2(r, c, robot.hp)
                else:
                    self._draw_grid_empty(r, c)

    def _draw_manual(self):
        r, c = self._manual_pos
        for line in self._manual_text.split("\n"):
            self._stdscr.addstr(r, c, line, self._attr_text)
            r += 1

    def _draw_cell_info(self):
        state = self._game.get_state(self._turn)
        actions = self._game._actions_on_turn[self._turn]
        r, c = self._selected
        s = "Selected: " + str((r, c))
        if (r, c) in settings.obstacles:
            s += "\nObstacle"
        elif state.is_robot((r, c)):
            robot = state.robots[(r, c)]
            s += "\nRobot:  " + self._names[robot.player_id]
            s += "\nHP:     " + str(robot.hp)
            s += "\nAction: " + actions[(r, c)]['name']
            if actions[(r, c)]['name'] in ['move', 'attack']:
                s += " " + str(actions[(r, c)]['target'])
        else:
            s += "\nEmpty"
        r, c = self._cell_info_pos
        for line in s.split("\n"):
            self._stdscr.addstr(r, c, line, self._attr_text)
            r += 1

    def _draw_score(self):
        state = self._game.get_state(self._turn)
        score = [0, 0]
        for bot in list(state.robots.values()):
            score[bot.player_id] += 1
        r, c = self._score_pos
        if score[0] >= score[1]:
            s = self._names[0]
            self._stdscr.addstr(r, c, s, self._attr_bot1)
            c += len(s)
            s = " " + str(score[0]) + " : " + str(score[1]) + " "
            self._stdscr.addstr(r, c, s, self._attr_text)
            c += len(s)
            s = self._names[1]
            self._stdscr.addstr(r, c, s, self._attr_bot2)
        else:
            s = self._names[1]
            self._stdscr.addstr(r, c, s, self._attr_bot2)
            c += len(s)
            s = " " + str(score[1]) + " : " + str(score[0]) + " "
            self._stdscr.addstr(r, c, s, self._attr_text)
            c += len(s)
            s = self._names[0]
            self._stdscr.addstr(r, c, s, self._attr_bot1)

    def _draw_final_score(self):
        state = self._game.get_state(settings.max_turns)
        score = [0, 0]
        for bot in list(state.robots.values()):
            score[bot.player_id] += 1
        r, c = self._final_score_pos
        if score[0] > score[1]:
            s = self._names[0]
            self._stdscr.addstr(r, c, s, self._attr_bot1)
            c += len(s)
            s = " won: " + str(score[0]) + " : " + str(score[1])
            self._stdscr.addstr(r, c, s, self._attr_text)
        elif score[1] > score[0]:
            s = self._names[1]
            self._stdscr.addstr(r, c, s, self._attr_bot2)
            c += len(s)
            s = " won: " + str(score[1]) + " : " + str(score[0])
            self._stdscr.addstr(r, c, s, self._attr_text)
        else:
            s = "Tie game: " + str(score[0]) + " : " + str(score[1])
            self._stdscr.addstr(r, c, s, self._attr_text)

    def _draw_turn(self):
        r, c = self._turn_pos
        s = "Turn: " + str(self._turn)
        self._stdscr.addstr(r, c, s, self._attr_text)

    def _draw_screen(self):
        self._draw_game_grid()
        self._draw_manual()
        self._draw_cell_info()
        self._draw_score()
        self._draw_final_score()
        self._draw_turn()

    def _increase_turn(self):
        if self._turn < settings.max_turns:
            self._turn += 1
            return True
        else:
            return False

    def _decrease_turn(self):
        if self._turn > 0:
            self._turn -= 1
            return True
        else:
            return False

    def _move_selected_up(self):
        self._selected[1] = max(self._selected[1] - 1, 0)

    def _move_selected_left(self):
        self._selected[0] = max(self._selected[0] - 1, 0)

    def _move_selected_down(self):
        self._selected[1] = min(self._selected[1] + 1,
                                settings.board_size - 1)

    def _move_selected_right(self):
        self._selected[0] = min(self._selected[0] + 1,
                                settings.board_size - 1)

    def _handle_key(self, key):
        if key in self._pause_keys:
            self._paused = not self._paused
        elif key in self._exit_keys:
            self._done = True
        elif key in self._step_keys:
            self._increase_turn()
            self._paused = True
        elif key in self._back_keys:
            self._decrease_turn()
            self._paused = True
        elif key in self._rewind_keys:
            self._turn = 0
            self._paused = True
        elif key in self._up_keys:
            self._move_selected_up()
            self._paused = True
        elif key in self._left_keys:
            self._move_selected_left()
            self._paused = True
        elif key in self._down_keys:
            self._move_selected_down()
            self._paused = True
        elif key in self._right_keys:
            self._move_selected_right()
            self._paused = True

    def _main_loop(self):
        self._done = False
        self._turn = 0
        while not self._done:
            if self._paused:
                # When paused, block until a key is pressed
                self._stdscr.timeout(-1)
                c = self._stdscr.getch()
                self._handle_key(c)
            else:
                # When playing, block until a key a pressed or time to
                # advance to the next turn. Since it unblocks on ANY key press,
                # pressing random keys when the game is playing will prevent
                # it from advancing to the next turn for a short while. A
                # solution might be to use the time module to only wait the
                # time difference from the last turn advance, but this may
                # introduce cross-platform issues due to inconsistencies in
                # time resolution.
                self._stdscr.timeout(self._turn_delay)
                c = self._stdscr.getch()
                if c == cs.ERR:
                    if not self._increase_turn():
                        self._paused = True
                else:
                    self._handle_key(c)
            # TODO: Not every cell need to be cleared. Clearing everything
            #       creates flicker.
            self._stdscr.clear()
            self._draw_screen()
            self._stdscr.refresh()

    def _main(self, stdscr):
        self._init_curses()
        self._stdscr = stdscr

        # Start loop
        self._main_loop()




import random
from collections import defaultdict

from rgkit import rg
from rgkit.settings import settings, AttrDict


class GameState(object):
    def __init__(self, use_start=False, turn=0,
                 next_robot_id=0, seed=None, symmetric=True):
        if seed is None:
            seed = random.randint(0, settings.max_seed)
        self._seed = str(seed)
        self._spawn_random = random.Random(self._seed + 's')
        self._attack_random = random.Random(self._seed + 'a')

        self.robots = {}
        self.turn = turn
        self._next_robot_id = next_robot_id

        if use_start and settings.start is not None:
            for i, start in enumerate(settings.start):
                for loc in start:
                    self.add_robot(loc, i)

        self.symmetric = symmetric
        if symmetric:
            assert settings.player_count == 2
            self._get_spawn_locations = self._get_spawn_locations_symmetric
        else:
            self._get_spawn_locations = self._get_spawn_locations_random

    @staticmethod
    def _loc_from_json(location):
        return (int(location[0]), int(location[1]))

    @classmethod
    def create_from_json(cls, data):
        state = GameState(turn=int(data['turn']), seed=data['seed'])
        for bot in data['robots']:
            state.add_robot(bot['location'], bot['player_id'], bot['hp'],
                            bot['robot_id'])
        return state

    @classmethod
    def create_actions_from_json(cls, actions):
        moves = {}
        for action in actions:
            loc = cls._loc_from_json(action['location'])
            move = action['action']
            if len(move) > 1:
                move[1] = cls._loc_from_json(move[1])
            moves[loc] = move
        return moves

    def add_robot(self, loc, player_id, hp=None, robot_id=None):
        if hp is None:
            hp = settings.robot_hp

        if robot_id is None:
            robot_id = self._next_robot_id
            self._next_robot_id += 1

        self.robots[self._loc_from_json(loc)] = AttrDict({
            'location': self._loc_from_json(loc),
            'hp': int(hp),
            'player_id': int(player_id),
            'robot_id': int(robot_id),
        })

    def remove_robot(self, loc):
        if self.is_robot(loc):
            del self.robots[loc]

    def is_robot(self, loc):
        return loc in self.robots

    def _get_spawn_locations_symmetric(self):
        def symmetric_loc(loc):
            return (settings.board_size - 1 - loc[0],
                    settings.board_size - 1 - loc[1])
        locs1 = []
        locs2 = []
        while len(locs1) < settings.spawn_per_player:
            loc = self._spawn_random.choice(settings.spawn_coords)
            sloc = symmetric_loc(loc)
            if loc not in locs1 and loc not in locs2:
                if sloc not in locs1 and sloc not in locs2:
                    locs1.append(loc)
                    locs2.append(sloc)
        return locs1 + locs2

    def _get_spawn_locations_random(self):
        # see http://stackoverflow.com/questions/2612648/reservoir-sampling
        locations = []
        per_player = settings.spawn_per_player
        count = per_player * settings.player_count
        n = 0
        for loc in settings.spawn_coords:
            n += 1
            if len(locations) < count:
                locations.append(loc)
            else:
                s = int(self._spawn_random.random() * n)
                if s < count:
                    locations[s] = loc
        self._spawn_random.shuffle(locations)
        return locations

    def _get_contenders(self, dest):
        """
        Generates a dict of locations, where the values correspond to the
        set of bots that wish to move into that square or will be moving
        into that square. This is because due to collisions a bot can
        'use up' two squares:
        1. the first is blocked because he attempted to move into it
        2. the second is blocked because that is his current location
           where he will be staying due to the collision at 1

        :param dest: func(location of robot) = destination of robot
        :returns: dict[destination] = set(locations of bots that either
                                        want to move to 'destination' or
                                        are moving to 'destination'
                                        because of collisions)
        """
        contenders = defaultdict(lambda: set())

        def stuck(loc):
            # Robot at loc is stuck
            # Other robots trying to move in its old locations
            # should be marked as stuck, too
            old_contenders = contenders[loc]
            contenders[loc] = set([loc])

            for contender in old_contenders:
                if contender != loc:
                    stuck(contender)

        for loc in self.robots:
            contenders[dest(loc)].add(loc)

        for loc in self.robots:
            if len(contenders[dest(loc)]) > 1 or (self.is_robot(dest(loc)) and
                                                  dest(loc) != loc and
                                                  dest(dest(loc)) == loc):
                # Robot at loc is going to fail to move
                stuck(loc)

        return contenders

    # new_locations = {loc: new_loc}
    def _get_new_locations(self, dest, contenders):
        new_locations = {}

        for loc in self.robots:
            if loc != dest(loc) and loc in contenders[loc]:
                new_locations[loc] = loc
            else:
                new_locations[loc] = dest(loc)

        return new_locations

    # collisions = {loc: set(robots collided with robot at loc)}
    def _get_collisions(self, dest, contenders):
        collisions = defaultdict(lambda: set())

        for loc in self.robots:
            for loc2 in contenders[dest(loc)]:
                collisions[loc].add(loc2)
                collisions[loc2].add(loc)

        return collisions

    # damage_map = {loc: [actor_id: (actor_loc, damage)]}
    # only counts potential attack and suicide damage
    # self suicide damage is not counted
    def _get_damage_map(self, actions):
        damage_map = defaultdict(
            lambda: [{} for _ in range(settings.player_count)])

        for loc, robot in list(self.robots.items()):
            actor_id = robot.player_id

            if actions[loc][0] == 'attack':
                target = actions[loc][1]
                damage = self._attack_random.randint(
                    *settings.attack_range)
                damage_map[target][actor_id][loc] = damage
            elif actions[loc][0] == 'suicide':
                damage = settings.suicide_damage
                for target in rg.locs_around(loc):
                    damage_map[target][actor_id][loc] = damage

        return damage_map

    @staticmethod
    def _apply_damage_caused(delta, damage_caused):
        for robot_delta in delta:
            robot_delta.damage_caused += damage_caused[robot_delta.loc]

    @staticmethod
    def _apply_spawn(delta, spawn_locations):
        # clear robots on spawn
        for robot_delta in delta:
            if robot_delta.loc_end in settings.spawn_coords:
                robot_delta.hp_end = 0

        # spawn robots
        for i in range(settings.spawn_per_player):
            for player_id in range(settings.player_count):
                loc = spawn_locations[player_id*settings.spawn_per_player+i]
                delta.append(AttrDict({
                    'loc': loc,
                    'hp': 0,
                    'player_id': player_id,
                    'loc_end': loc,
                    'hp_end': settings.robot_hp,
                    'damage_caused': 0
                }))

    # actions = {loc: action}
    # all actions must be valid
    # delta = [AttrDict{
    #    'loc': loc,
    #    'hp': hp,
    #    'player_id': player_id,
    #    'loc_end': loc_end,
    #    'hp_end': hp_end
    #    'damage_caused': damage_caused
    # }]
    def get_delta(self, actions, spawn=True):
        delta = []

        def dest(loc):
            if actions[loc][0] == 'move':
                return actions[loc][1]
            else:
                return loc

        contenders = self._get_contenders(dest)
        new_locations = self._get_new_locations(dest, contenders)
        collisions = self._get_collisions(dest, contenders)
        damage_map = self._get_damage_map(actions)
        damage_caused = defaultdict(lambda: 0)  # {loc: damage_caused}

        for loc, robot in list(self.robots.items()):
            robot_delta = AttrDict({
                'loc': loc,
                'hp': robot.hp,
                'player_id': robot.player_id,
                'loc_end': new_locations[loc],
                'hp_end': robot.hp,  # to be adjusted
                'damage_caused': 0  # to be adjusted
            })

            is_guard = actions[loc][0] == 'guard'

            # collision damage
            if not is_guard:
                damage = settings.collision_damage

                for other_loc in collisions[loc]:
                    if robot.player_id != self.robots[other_loc].player_id:
                        robot_delta.hp_end -= damage
                        damage_caused[other_loc] += damage

            # attack and suicide damage
            for player_id, player_damage_map in enumerate(
                    damage_map[new_locations[loc]]):
                if player_id != robot.player_id:
                    for actor_loc, damage in list(player_damage_map.items()):
                        if is_guard:
                            damage //= 2

                        robot_delta.hp_end -= damage
                        damage_caused[actor_loc] += damage

            # account for suicides
            if actions[loc][0] == 'suicide':
                robot_delta.hp_end = 0

            delta.append(robot_delta)

        self._apply_damage_caused(delta, damage_caused)

        if spawn and self.turn % settings.spawn_every == 0:
            self._apply_spawn(delta, self._get_spawn_locations())

        return delta

    # delta = [AttrDict{
    #    'loc': loc,
    #    'hp': hp,
    #    'player_id': player_id,
    #    'loc_end': loc_end,
    #    'hp_end': hp_end,
    #    'damage_caused': damage_caused
    # }]
    # returns new GameState
    def apply_delta(self, delta):
        new_state = GameState(settings,
                              next_robot_id=self._next_robot_id,
                              turn=self.turn + 1,
                              seed=self._spawn_random.randint(
                                  0, settings.max_seed),
                              symmetric=self.symmetric)

        for delta_info in delta:
            if delta_info.hp_end > 0:
                loc = delta_info.loc

                # is this a new robot?
                if delta_info.hp > 0:
                    robot_id = self.robots[loc].robot_id
                else:
                    robot_id = None

                new_state.add_robot(delta_info.loc_end, delta_info.player_id,
                                    delta_info.hp_end, robot_id)

        return new_state

    # actions = {loc: action}
    # all actions must be valid
    # returns new GameState
    def apply_actions(self, actions, spawn=True):
        delta = self.get_delta(actions, spawn)

        return self.apply_delta(delta)

    def get_scores(self):
        scores = [0 for _ in range(settings.player_count)]

        for robot in list(self.robots.values()):
            scores[robot.player_id] += 1

        return scores

    # export GameState to be used by a robot
    def get_game_info(self, player_id=None, json=None, seed=None):
        game_info = AttrDict()

        if json:
            game_info.robots = list(self.robots.values())
        else:
            game_info.robots = dict((loc, AttrDict(robot))
                                    for loc, robot in list(self.robots.items()))

        if player_id is not None:
            for robot in list(game_info.robots.values()):
                if robot.player_id != player_id:
                    del robot.robot_id

        if seed:
            game_info.seed = self._seed

        game_info.turn = self.turn
        if json:
            game_info.turn = str(game_info.turn)

        return game_info



import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit.gamestate import GameState

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestSpawn(unittest.TestCase):
    def test_first_turn_spawning(self):
        state = GameState()

        actions = {}

        state2 = state.apply_actions(actions)

        self.assertEqual(state2.get_scores(),
                         [settings.spawn_per_player,
                          settings.spawn_per_player])

    def test_spawn_kills(self):
        state = GameState()

        state.add_robot((4, 3), 0)

        actions = {
            (4, 3): ['guard']
        }

        state2 = state.apply_actions(actions)

        self.assertTrue(not state2.is_robot((4, 3)) or
                        state2.robots[(4, 3)].robot_id !=
                        state.robots[4, 3].robot_id)

    def test_spawn_dodge(self):
        state = GameState()

        state.add_robot((4, 3), 0)

        actions = {
            (4, 3): ['move', (4, 4)]
        }

        state2 = state.apply_actions(actions)

        self.assertTrue(state2.is_robot((4, 4)))

    def test_spawn_dodge_overwrite(self):
        state = GameState()

        state.add_robot((4, 3), 0)

        actions = {
            (4, 3): ['move', (4, 4)]
        }

        # ensure that a robot will get spawned at (4, 3)
        state._get_spawn_locations = lambda: [(i, 3) for i in range(10)]

        state2 = state.apply_actions(actions)

        self.assertTrue(state2.is_robot((4, 4)))
        self.assertTrue(state2.is_robot((4, 3)))

    def test_spawn_hop_in(self):
        state = GameState()

        state.add_robot((4, 4), 0)

        actions = {
            (4, 4): ['move', (4, 3)]
        }

        state2 = state.apply_actions(actions)

        self.assertTrue(not state2.is_robot((4, 3)) or
                        state2.robots[(4, 3)].robot_id !=
                        state.robots[4, 4].robot_id)



import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit.gamestate import GameState
from rgkit.game import Player

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestValid(unittest.TestCase):
    def test_ok(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        Player._validate_action(robot, ['guard'])
        Player._validate_action(robot, ['suicide'])
        Player._validate_action(robot, ['guard', None])
        Player._validate_action(robot, ['suicide', (1, 1)])
        Player._validate_action(robot, ['attack', (9, 10)])
        Player._validate_action(robot, ['move', (9, 10)])
        Player._validate_action(robot, ('guard', (2, 3)))
        Player._validate_action(robot, ('suicide', None))
        Player._validate_action(robot, ('attack', (9, 10)))
        Player._validate_action(robot, ('move', (9, 10)))

    def test_none(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, None)

    def test_strange(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, "ALL YOUR BASE ARE BELONG TO US")

    def test_wrong_command(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['exterminate'])

    def test_lack_of_location(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]

        with self.assertRaises(Exception):
            Player._validate_action(robot, ['move'])

    def test_move_to_self(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['move', (9, 9)])

    def test_move_too_far(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['move', (9, 11)])

    def test_move_to_obstacle(self):
        state = GameState()
        state.add_robot((2, 5), 0)
        robot = state.robots[(2, 5)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['move', (2, 4)])

    def test_attack_self(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['attack', (9, 9)])

    def test_attack_too_far(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        robot = state.robots[(9, 9)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['attack', (9, 11)])

    def test_attack_obstacle(self):
        state = GameState()
        state.add_robot((2, 5), 0)
        robot = state.robots[(2, 5)]
        with self.assertRaises(Exception):
            Player._validate_action(robot, ['attack', (2, 4)])





import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit.gamestate import GameState

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestSuicide(unittest.TestCase):
    def test_basic_suicide(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)

        actions = {
            (10, 10): ['attack', (9, 10)],
            (9, 10): ['suicide']
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp,
                         settings.robot_hp - settings.suicide_damage)

        self.assertFalse(state2.is_robot((9, 10)))

    def test_guarding_reduces_damage(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)

        actions = {
            (10, 10): ['guard'],
            (9, 10): ['suicide']
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp,
                         settings.robot_hp - settings.suicide_damage//2)
        self.assertFalse(state2.is_robot((9, 10)))

    def test_suicide_does_no_damage_to_teammates(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 0)

        actions = {
            (10, 10): ['guard'],
            (9, 10): ['suicide']
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp, settings.robot_hp)
        self.assertFalse(state2.is_robot((9, 10)))

    def test_suicide_dodge(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)

        actions = {
            (10, 10): ['sucide'],
            (9, 10): ['move', (8, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((8, 10)))
        self.assertEqual(state2.robots[8, 10].hp, settings.robot_hp)

    def test_suicide_dodge_fail(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)
        state.add_robot((7, 10), 1)

        actions = {
            (10, 10): ['suicide'],
            (9, 10): ['move', (8, 10)],
            (7, 10): ['move', (8, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp,
                         settings.robot_hp - settings.suicide_damage)

    def test_move_into_suicide_bot(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)

        actions = {
            (10, 10): ['suicide'],
            (9, 10): ['move', (10, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp,
                         settings.robot_hp -
                         settings.collision_damage -
                         settings.suicide_damage)



#!/usr/bin/env python
'''
Run: ./setup.py register sdist upload
Check: ./setup.py checkdocs (pip install collective.checkdocs)
'''

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    long_description = readme_file.read()

setup(
    name='rgkit_py3',
    version='1.1.1',
    description='Game Engine for Robot Game',
    maintainer='Aristeidis Tomaras',
    maintainer_email='arisgold29@gmail.com',
    url='https://github.com/saavedra29/rgkit_py3',
    packages=find_packages(),
    package_data={'rgkit': ['bots/*.py', 'maps/*.py']},
    license='Unlicense',
    long_description=long_description,
    entry_points={
        'console_scripts': [
            'rgrun = rgkit.run:main',
            'rgmap = rgkit.mapeditor:main'
        ]
    },
)



#!/usr/bin/env python2



import ast
import tkinter
import sys
import os.path

from rgkit.settings import settings

BLOCKSIZE = 20
PADDING = 4

color_mapping = {
    'a': ('black', 'obstacle'),
    's': ('#ddd', None),
    'g': ('darkgreen', 'spawn'),
    't': ('pink', 'start1'),
    'h': ('lightgreen', 'start2'),
    'r': ('darkred', None),
}


def print_instructions():
    print('''
usage: python mapeditor.py <starting map file>

I made this map editor to use for myself. Therefore, it might not seem
very user-friendly, but it's not that hard. There are just a bunch of
keyboard shortcuts to use.

Game-related colors
===================
[a] paint black (obstacle)
[s] erase (walking space)
[g] paint green (spawn point)
[t] paint pink (starting robot for p1)
[h] paint light green (starting robot for p2)

Just for yourself
=================
[r] paint red

Other functions
===============
[d] fill board with selected color
[f] save map data to map file provided
[i] invert black and white colors
''')


class MapEditor(object):
    def __init__(self, blocksize, padding, map_file):
        self._blocksize = blocksize
        self._padding = padding
        self._map_file = map_file
        self.make_canvas()

    def make_canvas(self):
        root = tkinter.Tk()
        size = (self._blocksize + self._padding) * settings.board_size + \
            self._padding * 2 + 40

        self._canvas = tkinter.Canvas(root, width=size, height=size)
        self._rect_items = []
        self._colors = []
        self._pressed = False

        self.prepare_backdrop(size)
        self.load_map()
        self.set_color('black')
        self.bind_events()

        self._canvas.pack()
        root.title('Robot Game Map Editor')
        root.mainloop()

    def prepare_backdrop(self, size):
        for y in range(settings.board_size):
            for x in range(settings.board_size):
                item = self._canvas.create_rectangle(
                    x * (self._blocksize + self._padding) + self._padding + 20,
                    y * (self._blocksize + self._padding) + self._padding + 20,
                    (x + 1) * (self._blocksize + self._padding) + 20,
                    (y + 1) * (self._blocksize + self._padding) + 20,
                    fill='#ddd',
                    width=0)
                self._colors.append('#ddd')
                self._rect_items.append(item)
        self._bar = self._canvas.create_rectangle(0, 0, size, 15, width=0)

    def set_color(self, color):
        self._current_color = color
        self._canvas.itemconfigure(self._bar, fill=self._current_color)

    def bind_events(self):
        self._canvas.bind_all('<Button-1>', self.click_handler)
        self._canvas.bind_all('<B1-Motion>', self.move_handler)
        self._canvas.bind_all('<ButtonRelease-1>', self.release_handler)
        self._canvas.bind_all('<Key>', self.key_handler)

    def paint_square(self, tk_event=None, item_id=None):
        if tk_event is None and item_id is None:
            raise Exception('must supply either tk_event or item_id')

        if item_id is None:
            try:
                widget = tk_event.widget.find_closest(tk_event.x, tk_event.y)
                item_id = widget[0] - 1
            except:
                return

        if 0 <= item_id < len(self._rect_items):
            self._canvas.itemconfigure(self._rect_items[item_id],
                                       fill=self._current_color)
            self._colors[item_id] = self._current_color

    def click_handler(self, event):
        self._pressed = True
        self.paint_square(tk_event=event)

    def move_handler(self, event):
        if self._pressed:
            self.paint_square(tk_event=event)

    def release_handler(self, _):
        self._pressed = False

    def paint_all(self):
        for i, item in enumerate(self._rect_items):
            self.paint_square(item_id=i)
            self._canvas.itemconfigure(item, fill=self._current_color)

    def load_map(self):
        if self._map_file is None:
            return
        if not os.path.exists(self._map_file):
            map_data = {'spawn': [], 'obstacle': []
                        }
        else:
            map_data = ast.literal_eval(open(self._map_file).read())

        label_mapping = dict((v, k) for k, v in list(color_mapping.values())
                             if v is not None)

        for label, color in list(label_mapping.items()):
            if label not in map_data:
                continue

            self.set_color(color)
            for coord in map_data[label]:
                self.paint_square(item_id=coord[0] +
                                  settings.board_size * coord[1])

    def save_map(self):
        coords = {}
        label_mapping = dict(list(color_mapping.values()))

        for color, label in list(label_mapping.items()):
            if label is not None:
                coords[label] = []

        for i, color in enumerate(self._colors):
            if color in label_mapping and label_mapping[color] is not None:
                coords[label_mapping[color]].append((i % settings.board_size,
                                                     i // settings.board_size))

        with open(self._map_file, 'w') as f:
            f.write(str(coords))
            print('saved!')

    def invert_colors(self):
        old_color = self._current_color
        for i, c in enumerate(self._colors):
            if c == '#ddd':
                self._current_color = 'black'
            elif c == 'black':
                self._current_color = '#ddd'

            self.paint_square(item_id=i)
            self._colors[i] = self._current_color
        self._current_color = old_color

    def key_handler(self, event):
        if event.char in color_mapping:
            self.set_color(color_mapping[event.char][0])

        func_map = {
            'd': self.paint_all,
            'f': self.save_map,
            'i': self.invert_colors
        }

        if event.char in func_map:
            func_map[event.char]()


def main():
    if len(sys.argv) > 1:
        MapEditor(BLOCKSIZE, PADDING, sys.argv[1])
    else:
        print_instructions()

if __name__ == '__main__':
    main()



import ast
import pkg_resources
import unittest

from rgkit.settings import settings
from rgkit.gamestate import GameState

map_data = ast.literal_eval(
    open(pkg_resources.resource_filename('rgkit', 'maps/default.py')).read())
settings.init_map(map_data)


class TestMove(unittest.TestCase):
    def test_move_no_collision(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((8, 10), 0)

        actions = {
            (10, 10): ['move', (11, 10)],
            (8, 10): ['move', (7, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((11, 10)))
        self.assertEqual(state2.robots[11, 10].hp, settings.robot_hp)
        self.assertTrue(state2.is_robot((7, 10)))
        self.assertEqual(state2.robots[7, 10].hp, settings.robot_hp)

    def test_basic_collision(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((8, 10), 1)

        actions = {
            (10, 10): ['move', (9, 10)],
            (8, 10): ['move', (9, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((8, 10)))
        self.assertEqual(state2.robots[8, 10].hp,
                         settings.robot_hp - settings.collision_damage)

    def test_guarding_ingnores_collisions(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((9, 10), 1)

        actions = {
            (10, 10): ['move', (9, 10)],
            (9, 10): ['guard']
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp, settings.robot_hp)

    def test_move_train(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((11, 10), 0)
        state.add_robot((12, 10), 0)

        actions = {
            (10, 10): ['move', (9, 10)],
            (11, 10): ['move', (10, 10)],
            (12, 10): ['move', (11, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp, settings.robot_hp)
        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp, settings.robot_hp)
        self.assertTrue(state2.is_robot((11, 10)))
        self.assertEqual(state2.robots[11, 10].hp, settings.robot_hp)

    def test_train_collision(self):
        state = GameState()
        state.add_robot((10, 10), 0)
        state.add_robot((11, 10), 1)
        state.add_robot((12, 10), 0)
        state.add_robot((8, 10), 1)

        actions = {
            (10, 10): ['move', (9, 10)],
            (11, 10): ['move', (10, 10)],
            (12, 10): ['move', (11, 10)],
            (8, 10): ['move', (9, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((10, 10)))
        self.assertEqual(state2.robots[10, 10].hp,
                         settings.robot_hp - settings.collision_damage*2)
        self.assertTrue(state2.is_robot((11, 10)))
        self.assertEqual(state2.robots[11, 10].hp,
                         settings.robot_hp - settings.collision_damage*2)
        self.assertTrue(state2.is_robot((12, 10)))
        self.assertEqual(state2.robots[12, 10].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((8, 10)))
        self.assertEqual(state2.robots[8, 10].hp,
                         settings.robot_hp - settings.collision_damage)

    def test_try_swap(self):
        state = GameState()
        state.add_robot((9, 9), 0)
        state.add_robot((8, 9), 1)

        actions = {
            (9, 9): ['move', (8, 9)],
            (8, 9): ['move', (9, 9)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 9)))
        self.assertEqual(state2.robots[9, 9].player_id, 0)
        self.assertEqual(state2.robots[9, 9].robot_id,
                         state.robots[9, 9].robot_id)
        self.assertEqual(state2.robots[9, 9].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((8, 9)))
        self.assertEqual(state2.robots[8, 9].player_id, 1)
        self.assertEqual(state2.robots[8, 9].robot_id,
                         state.robots[8, 9].robot_id)
        self.assertEqual(state2.robots[8, 9].hp,
                         settings.robot_hp - settings.collision_damage)

    def test_try_move_in_circle(self):
        state = GameState()
        state.add_robot((8, 8), 0)
        state.add_robot((9, 8), 1)
        state.add_robot((9, 9), 0)
        state.add_robot((8, 9), 1)

        actions = {
            (8, 8): ['move', (9, 8)],
            (9, 8): ['move', (9, 9)],
            (9, 9): ['move', (8, 9)],
            (8, 9): ['move', (8, 8)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 8)))
        self.assertEqual(state2.robots[9, 8].hp, settings.robot_hp)
        self.assertEqual(state2.robots[9, 8].player_id, 0)
        self.assertTrue(state2.is_robot((9, 9)))
        self.assertEqual(state2.robots[9, 9].hp, settings.robot_hp)
        self.assertEqual(state2.robots[9, 9].player_id, 1)
        self.assertTrue(state2.is_robot((8, 9)))
        self.assertEqual(state2.robots[8, 9].hp, settings.robot_hp)
        self.assertEqual(state2.robots[8, 9].player_id, 0)
        self.assertTrue(state2.is_robot((8, 8)))
        self.assertEqual(state2.robots[8, 8].hp, settings.robot_hp)
        self.assertEqual(state2.robots[8, 8].player_id, 1)

    def test_unobvious_collision(self):
        state = GameState()
        state.add_robot((9, 10), 0)
        state.add_robot((9, 12), 1)
        state.add_robot((9, 11), 0)
        state.add_robot((11, 11), 1)

        actions = {
            (9, 10): ['move', (9, 11)],
            (9, 12): ['move', (9, 11)],
            (9, 11): ['move', (10, 11)],
            (11, 11): ['move', (10, 11)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp, settings.robot_hp)
        self.assertTrue(state2.is_robot((9, 12)))
        self.assertEqual(state2.robots[9, 12].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((9, 11)))
        self.assertEqual(state2.robots[9, 11].hp,
                         settings.robot_hp - settings.collision_damage*2)
        self.assertTrue(state2.is_robot((11, 11)))
        self.assertEqual(state2.robots[11, 11].hp,
                         settings.robot_hp - settings.collision_damage)

    def test_double_collision(self):
        state = GameState()
        state.add_robot((9, 10), 0)
        state.add_robot((10, 11), 0)
        state.add_robot((11, 10), 1)

        actions = {
            (9, 10): ['move', (10, 10)],
            (10, 11): ['move', (10, 10)],
            (11, 10): ['move', (10, 10)]
        }

        state2 = state.apply_actions(actions, spawn=False)

        self.assertTrue(state2.is_robot((9, 10)))
        self.assertEqual(state2.robots[9, 10].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((10, 11)))
        self.assertEqual(state2.robots[10, 11].hp,
                         settings.robot_hp - settings.collision_damage)
        self.assertTrue(state2.is_robot((11, 10)))
        self.assertEqual(state2.robots[11, 10].hp,
                         settings.robot_hp - settings.collision_damage*2)



import json
import random
import unittest

from rgkit.gamestate import GameState
from rgkit.settings import settings


class TestJson(unittest.TestCase):
    game1 = {
        'seed': '0',
        'turn': '12',
        'robots': [
            {
                'location': ['10', '12'],
                'hp': '50',
                'player_id': '0',
                'robot_id': '5',
            },
            {
                'location': ['10', '10'],
                'hp': '50',
                'player_id': '1',
                'robot_id': '6',
            }
        ],
    }

    actions1 = [
        {
            'location': ['10', '12'],
            'action': ['move', ['10', '11']],
        },
        {
            'location': ['10', '10'],
            'action': ['attack', ['10', '11']],
        }
    ]

    result1 = {
        'seed': '----',  # Randomized.
        'turn': '13',
        'robots': [
            {
                'location': ['10', '11'],
                'hp': '----',  # Randomized.
                'player_id': '0',
                'robot_id': '5',
            },
            {
                'location': ['10', '10'],
                'hp': '50',
                'player_id': '1',
                'robot_id': '6',
            }
        ],
    }

    def sprint(self, val):
        # Oh man why, oh why.
        def to_str(data):
            if isinstance(data, dict):
                for k, v in list(data.items()):
                    data[k] = to_str(v)
            elif isinstance(data, list) or isinstance(data, tuple):
                new_data = []
                for v in data:
                    new_data.append(to_str(v))
                data = new_data
            else:
                data = str(data)
            return data
        return json.dumps(to_str(val), sort_keys=True)

    def test_load_json_game_state(self):
        # Dump and load to validate
        data = json.loads(json.dumps(self.game1))
        state = GameState.create_from_json(data)
        info = state.get_game_info(json=True, seed=True)

        self.assertEqual(self.sprint(self.game1), self.sprint(info))

    def test_load_json_actions(self):
        data = json.loads(json.dumps(self.actions1))
        moves = GameState.create_actions_from_json(data)

        self.assertTrue((10, 12) in moves)
        self.assertEqual(json.dumps(moves[(10, 12)]),
                         json.dumps(['move', [10, 11]]))

    def test_apply_actions(self):
        state = GameState.create_from_json(self.game1)
        moves = GameState.create_actions_from_json(self.actions1)
        new_state = state.apply_actions(moves)
        info = new_state.get_game_info(json=True, seed=True)

        # TODO: Less mysterious randomization please -_-.
        r1 = random.Random('0' + 's')
        self.result1['seed'] = str(r1.randint(0, settings.max_seed))
        r2 = random.Random('0' + 'a')
        hp_left = 50 - r2.randint(*settings.attack_range)
        self.result1['robots'][0]['hp'] = hp_left

        self.assertEqual(self.sprint(self.result1), self.sprint(info))



from rgkit.render.settings import settings as render_settings
from rgkit.render.utils import rgb_to_hex, blend_colors, compute_color


class HighlightSprite(object):
    def __init__(self, loc, target, render):
        self.location = loc
        self.target = target
        self.renderer = render
        self.hlt_square = None
        self.target_square = None

    def get_bot_color(self, loc):
        display_turn = self.renderer.current_turn_int()
        display_state = self.renderer._game.get_state(display_turn)
        if display_state.is_robot(loc):
            robot = display_state.robots[loc]
            bot_action = self.renderer._game.get_actions_on_turn(
                display_turn)[loc]['name']
            robot_color = compute_color(robot.player_id, robot.hp,
                                        bot_action)
            return robot_color
        return None

    def get_mixed_color(self, color, loc):
        bot_color = self.get_bot_color(loc)
        if bot_color is not None:
            color = blend_colors(color, bot_color, 0.7)
        return rgb_to_hex(*color)

    def clear_target_square(self):
        self.renderer.remove_object(self.target_square)
        self.target_square = None

    def clear(self):
        self.renderer.remove_object(self.hlt_square)
        self.hlt_square = None
        self.clear_target_square()

    def animate(self, delta=0):
        if render_settings.highlight_cursor_blink:
            if not delta < render_settings.highlight_cursor_blink_interval:
                self.clear()
                return

        if self.location is not None:
            if self.hlt_square is None:
                color = render_settings.highlight_color
                color = self.get_mixed_color(color, self.location)
                self.hlt_square = self.renderer.draw_grid_object(
                    self.location, fill=color, layer=3, width=0)
            if not self.renderer.show_arrows.get():
                if self.target is not None and self.target_square is None:
                    color = render_settings.target_color
                    color = self.get_mixed_color(color, self.target)
                    self.target_square = self.renderer.draw_grid_object(
                        self.target, fill=color, layer=3, width=0)
            else:
                self.clear_target_square()
